@@ -17,8 +17,8 @@
  * limitations under the License.
  */
 
-#include "vk_layer_utils.h"
-#include "vk_enum_string_helper.h"
+#include "utils/vk_layer_utils.h"
+#include "generated/vk_enum_string_helper.h"
 #include "core_checks/core_validation.h"
 
 #ifdef AHB_VALIDATION_SUPPORT
