@@ -21,15 +21,15 @@
 #include "state_tracker/base_node.h"
 #include "state_tracker/query_state.h"
 #include "state_tracker/video_session_state.h"
-#include "command_validation.h"
-#include "hash_vk_types.h"
-#include "subresource_adapter.h"
+#include "generated/command_validation.h"
+#include "utils/hash_vk_types.h"
+#include "containers/subresource_adapter.h"
 #include "state_tracker/image_layout_map.h"
 #include "state_tracker/pipeline_state.h"
 #include "state_tracker/device_state.h"
 #include "state_tracker/descriptor_sets.h"
-#include "qfo_transfer.h"
-#include "vk_layer_data.h"
+#include "containers/qfo_transfer.h"
+#include "containers/custom_containers.h"
 
 struct SUBPASS_INFO;
 class FRAMEBUFFER_STATE;
@@ -122,6 +122,11 @@ struct BufferBinding {
     VkDeviceSize stride;
 
     BufferBinding() : buffer_state(), size(0), offset(0), stride(0) {}
+    BufferBinding(const std::shared_ptr<BUFFER_STATE> &buffer_state_, VkDeviceSize size_, VkDeviceSize offset_,
+                  VkDeviceSize stride_)
+        : buffer_state(buffer_state_), size(size_), offset(offset_), stride(stride_) {}
+    BufferBinding(const std::shared_ptr<BUFFER_STATE> &buffer_state_, VkDeviceSize offset_)
+        : BufferBinding(buffer_state_, BUFFER_STATE::ComputeSize(buffer_state_, offset_, VK_WHOLE_SIZE), offset_, 0U) {}
     virtual ~BufferBinding() {}
 
     virtual void reset() { *this = BufferBinding(); }
@@ -132,6 +137,8 @@ struct IndexBufferBinding : BufferBinding {
     VkIndexType index_type;
 
     IndexBufferBinding() : BufferBinding(), index_type(static_cast<VkIndexType>(0)) {}
+    IndexBufferBinding(const std::shared_ptr<BUFFER_STATE> &buffer_state_, VkDeviceSize offset_, VkIndexType index_type_)
+        : BufferBinding(buffer_state_, offset_), index_type(index_type_) {}
     virtual ~IndexBufferBinding() {}
 
     virtual void reset() override { *this = IndexBufferBinding(); }
@@ -251,8 +258,8 @@ class CMD_BUFFER_STATE : public REFCOUNTED_NODE {
 
     // For each draw command D recorded to this command buffer, let
     //  * g_D be the graphics pipeline used
-    //  * v_G be the viewportCount of g_D (0 if g_D disables rasterization or enables VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT_EXT)
-    //  * s_G be the scissorCount  of g_D (0 if g_D disables rasterization or enables VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT_EXT)
+    //  * v_G be the viewportCount of g_D (0 if g_D disables rasterization or enables VK_DYNAMIC_STATE_VIEWPORT_WITH_COUNT)
+    //  * s_G be the scissorCount  of g_D (0 if g_D disables rasterization or enables VK_DYNAMIC_STATE_SCISSOR_WITH_COUNT)
     // Then this value is max(0, max(v_G for all D in cb), max(s_G for all D in cb))
     uint32_t usedViewportScissorCount;
     uint32_t pipelineStaticViewportCount;  // v_G for currently-bound graphics pipeline.
