@@ -142,6 +142,8 @@ cvdescriptorset::DescriptorClass cvdescriptorset::DescriptorTypeToClass(VkDescri
         case VK_DESCRIPTOR_TYPE_SAMPLED_IMAGE:
         case VK_DESCRIPTOR_TYPE_STORAGE_IMAGE:
         case VK_DESCRIPTOR_TYPE_INPUT_ATTACHMENT:
+        case VK_DESCRIPTOR_TYPE_SAMPLE_WEIGHT_IMAGE_QCOM:
+        case VK_DESCRIPTOR_TYPE_BLOCK_MATCH_IMAGE_QCOM:
             return Image;
         case VK_DESCRIPTOR_TYPE_UNIFORM_TEXEL_BUFFER:
         case VK_DESCRIPTOR_TYPE_STORAGE_TEXEL_BUFFER:
@@ -859,8 +861,9 @@ void cvdescriptorset::AccelerationStructureDescriptor::CopyUpdate(DescriptorSet
                                                                   const Descriptor *src, bool is_bindless) {
     if (src->GetClass() == Mutable) {
         auto acc_desc = static_cast<const MutableDescriptor *>(src);
+        is_khr_ = acc_desc->IsAccelerationStructureKHR();
         if (is_khr_) {
-            acc_ = acc_desc->GetAccelerationStructure();
+            acc_ = acc_desc->GetAccelerationStructureKHR();
             ReplaceStatePtr(set_state, acc_state_, dev_data->GetConstCastShared<ACCELERATION_STRUCTURE_STATE_KHR>(acc_),
                             is_bindless);
         } else {
@@ -1031,7 +1034,7 @@ void cvdescriptorset::MutableDescriptor::CopyUpdate(DescriptorSet *set_state, co
             } break;
             case AccelerationStructure: {
                 if (is_khr_) {
-                    acc_ = mutable_src->GetAccelerationStructure();
+                    acc_ = mutable_src->GetAccelerationStructureKHR();
                     ReplaceStatePtr(set_state, acc_state_, dev_data->GetConstCastShared<ACCELERATION_STRUCTURE_STATE_KHR>(acc_),
                                     is_bindless);
                 } else {
