@@ -19,35 +19,14 @@
 #include "state_tracker/pipeline_state.h"
 #include "state_tracker/descriptor_sets.h"
 #include "state_tracker/cmd_buffer_state.h"
-#include "enum_flag_bits.h"
+#include "generated/enum_flag_bits.h"
 
-static bool WrotePrimitiveShadingRate(VkShaderStageFlagBits stage_flag, const Instruction &entrypoint,
-                                      const SHADER_MODULE_STATE *module_state) {
-    bool primitive_rate_written = false;
-    if (stage_flag == VK_SHADER_STAGE_VERTEX_BIT || stage_flag == VK_SHADER_STAGE_GEOMETRY_BIT ||
-        stage_flag == VK_SHADER_STAGE_MESH_BIT_EXT) {
-        for (const Instruction *inst : module_state->GetBuiltinDecorationList()) {
-            if (inst->GetBuiltIn() == spv::BuiltInPrimitiveShadingRateKHR) {
-                primitive_rate_written = module_state->IsBuiltInWritten(inst, entrypoint);
-            }
-            if (primitive_rate_written) {
-                break;
-            }
-        }
-    }
-    return primitive_rate_written;
-}
-
-PipelineStageState::PipelineStageState(const safe_VkPipelineShaderStageCreateInfo *stage,
-                                       std::shared_ptr<const SHADER_MODULE_STATE> &module_state)
-    : module_state(module_state),
-      create_info(stage),
-      stage_flag(stage->stage),
-      entrypoint(module_state->FindEntrypoint(stage->pName, stage->stage)),
-      writes_to_gl_layer(module_state->WritesToGlLayer()) {
+PipelineStageState::PipelineStageState(const safe_VkPipelineShaderStageCreateInfo *create_info,
+                                       std::shared_ptr<const SHADER_MODULE_STATE> &module_state,
+                                       std::shared_ptr<const SHADER_MODULE_STATE::EntryPoint> &entrypoint)
+    : module_state(module_state), create_info(create_info), entrypoint(entrypoint) {
     if (entrypoint) {
-        descriptor_variables = module_state->GetResourceInterfaceVariable(*entrypoint);
-        wrote_primitive_shading_rate = WrotePrimitiveShadingRate(stage_flag, *entrypoint, module_state.get());
+        descriptor_variables = &(*entrypoint).resource_interface_variables;
     }
 }
 
@@ -84,59 +63,67 @@ PIPELINE_STATE::StageStateVec PIPELINE_STATE::GetStageStates(const ValidationSta
                     }
                 }
 
-                stage_states.emplace_back(&stage_ci, module);
+                auto entrypoint = module->FindEntrypoint(stage_ci.pName, stage_ci.stage);
+                stage_states.emplace_back(&stage_ci, module, entrypoint);
                 stage_found = true;
             }
         }
         if (!stage_found) {
             // Check if stage has been supplied by a library
+            std::shared_ptr<const SHADER_MODULE_STATE> module_state = nullptr;
+            const safe_VkPipelineShaderStageCreateInfo *stage_ci = nullptr;
             switch (stage) {
                 case VK_SHADER_STAGE_VERTEX_BIT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->vertex_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->vertex_shader_ci,
-                                                  pipe_state.pre_raster_state->vertex_shader);
+                        module_state = pipe_state.pre_raster_state->vertex_shader;
+                        stage_ci = pipe_state.pre_raster_state->vertex_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tessc_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->tessc_shader_ci,
-                                                  pipe_state.pre_raster_state->tessc_shader);
+                        module_state = pipe_state.pre_raster_state->tessc_shader;
+                        stage_ci = pipe_state.pre_raster_state->tessc_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->tesse_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->tesse_shader_ci,
-                                                  pipe_state.pre_raster_state->tesse_shader);
+                        module_state = pipe_state.pre_raster_state->tesse_shader;
+                        stage_ci = pipe_state.pre_raster_state->tesse_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_GEOMETRY_BIT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->geometry_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->geometry_shader_ci,
-                                                  pipe_state.pre_raster_state->geometry_shader);
+                        module_state = pipe_state.pre_raster_state->geometry_shader;
+                        stage_ci = pipe_state.pre_raster_state->geometry_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_TASK_BIT_EXT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->task_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->task_shader_ci,
-                                                  pipe_state.pre_raster_state->task_shader);
+                        module_state = pipe_state.pre_raster_state->task_shader;
+                        stage_ci = pipe_state.pre_raster_state->task_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_MESH_BIT_EXT:
                     if (pipe_state.pre_raster_state && pipe_state.pre_raster_state->mesh_shader) {
-                        stage_states.emplace_back(pipe_state.pre_raster_state->mesh_shader_ci,
-                                                  pipe_state.pre_raster_state->mesh_shader);
+                        module_state = pipe_state.pre_raster_state->mesh_shader;
+                        stage_ci = pipe_state.pre_raster_state->mesh_shader_ci;
                     }
                     break;
                 case VK_SHADER_STAGE_FRAGMENT_BIT:
                     if (pipe_state.fragment_shader_state && pipe_state.fragment_shader_state->fragment_shader) {
-                        stage_states.emplace_back(pipe_state.fragment_shader_state->fragment_shader_ci.get(),
-                                                  pipe_state.fragment_shader_state->fragment_shader);
+                        module_state = pipe_state.fragment_shader_state->fragment_shader;
+                        stage_ci = pipe_state.fragment_shader_state->fragment_shader_ci.get();
                     }
                     break;
                 default:
                     // no-op
                     break;
             }
+            if (!stage_ci) {
+                continue;
+            }
+            auto entrypoint = module_state->FindEntrypoint(stage_ci->pName, stage_ci->stage);
+            stage_states.emplace_back(stage_ci, module_state, entrypoint);
         }
     }
     return stage_states;
@@ -251,13 +238,20 @@ static bool UsesShaderModuleId(const PIPELINE_STATE &pipe_state) {
 
 static vvl::unordered_set<uint32_t> GetFSOutputLocations(const PIPELINE_STATE::StageStateVec &stage_states) {
     vvl::unordered_set<uint32_t> result;
-    for (const auto &stage : stage_states) {
-        if (!stage.entrypoint) {
+    for (const auto &stage_state : stage_states) {
+        if (!stage_state.entrypoint) {
             continue;
         }
-        if (stage.stage_flag == VK_SHADER_STAGE_FRAGMENT_BIT) {
-            result = stage.module_state->CollectWritableOutputLocationinFS(*(stage.entrypoint));
-            break;
+        if (stage_state.create_info->stage == VK_SHADER_STAGE_FRAGMENT_BIT) {
+            for (const auto *variable : stage_state.entrypoint->user_defined_interface_variables) {
+                if ((variable->storage_class != spv::StorageClassOutput) || variable->interface_slots.empty()) {
+                    continue;  // not an output interface
+                }
+                // It is not allowed to have Block Fragment or 64-bit vectors output in Frag shader
+                // This means all Locations in slots will be the same
+                result.insert(variable->interface_slots[0].Location());
+            }
+            break;  // found
         }
     }
     return result;
@@ -271,7 +265,7 @@ static VkPrimitiveTopology GetTopologyAtRasterizer(const PIPELINE_STATE &pipelin
         if (!stage.entrypoint) {
             continue;
         }
-        auto stage_topo = stage.module_state->GetTopology(*(stage.entrypoint));
+        auto stage_topo = stage.module_state->GetTopology(*stage.entrypoint);
         if (stage_topo) {
             result = *stage_topo;
         }
@@ -344,24 +338,9 @@ std::shared_ptr<FragmentShaderState> PIPELINE_STATE::CreateFragmentShaderState(
         }
     } else {
         if (lib_type == static_cast<VkGraphicsPipelineLibraryFlagsEXT>(0)) {  // Not a graphics library
-            bool contains_frag_shader = false;
-            const auto stages = vvl::make_span(create_info.pStages, create_info.stageCount);
-            for (const auto &stage : stages) {
-                if (stage.stage == VK_SHADER_STAGE_FRAGMENT_BIT) {
-                    contains_frag_shader = true;
-                    break;
-                }
-            }
-
             // No fragment shader _should_ imply no fragment shader state, however, for historical (GL) reasons, a pipeline _can_
-            // be created without a VS but no FS and still have valid fragment shader state. Here we try to infer if this is the
-            // case based on whether or not depth is written to.
+            // be created with a VS but no FS and still have valid fragment shader state.
             // See https://gitlab.khronos.org/vulkan/vulkan/-/issues/3178 for more details.
-            if (!contains_frag_shader && !(create_info.pDepthStencilState && create_info.pDepthStencilState->depthTestEnable &&
-                                           create_info.pDepthStencilState->depthWriteEnable)) {
-                return {};
-            }
-
             return std::make_shared<FragmentShaderState>(p, state, safe_create_info, rp);
         }
     }
@@ -558,6 +537,7 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       pipeline_type(VK_PIPELINE_BIND_POINT_GRAPHICS),
       create_flags(create_info.graphics.flags),
       shader_stages_ci(create_info.graphics.pStages, create_info.graphics.stageCount),
+      uses_shader_module_id(UsesShaderModuleId(*this)),
       vertex_input_state(CreateVertexInputState(*this, *state_data, create_info.graphics)),
       pre_raster_state(CreatePreRasterState(*this, *state_data, create_info.graphics, rpstate)),
       fragment_shader_state(CreateFragmentShaderState(*this, *state_data, *pCreateInfo, create_info.graphics, rpstate)),
@@ -570,7 +550,6 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
       topology_at_rasterizer(GetTopologyAtRasterizer(*this)),
-      uses_shader_module_id(UsesShaderModuleId(*this)),
       descriptor_buffer_mode((create_info.graphics.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
       csm_states(csm_states) {
@@ -641,12 +620,12 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       pipeline_type(VK_PIPELINE_BIND_POINT_COMPUTE),
       create_flags(create_info.compute.flags),
       shader_stages_ci(&create_info.compute.stage, 1),
+      uses_shader_module_id(UsesShaderModuleId(*this)),
       stage_states(GetStageStates(*state_data, *this, csm_states)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // compute has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
-      uses_shader_module_id(UsesShaderModuleId(*this)),
       descriptor_buffer_mode((create_info.compute.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
       csm_states(csm_states),
@@ -664,12 +643,12 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       create_flags(create_info.raytracing.flags),
       shader_stages_ci(create_info.raytracing.pStages, create_info.raytracing.stageCount),
       ray_tracing_library_ci(create_info.raytracing.pLibraryInfo),
+      uses_shader_module_id(UsesShaderModuleId(*this)),
       stage_states(GetStageStates(*state_data, *this, csm_states)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
-      uses_shader_module_id(UsesShaderModuleId(*this)),
       descriptor_buffer_mode((create_info.raytracing.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
       csm_states(csm_states),
@@ -689,12 +668,12 @@ PIPELINE_STATE::PIPELINE_STATE(const ValidationStateTracker *state_data, const V
       create_flags(create_info.raytracing.flags),
       shader_stages_ci(create_info.raytracing.pStages, create_info.raytracing.stageCount),
       ray_tracing_library_ci(create_info.raytracing.pLibraryInfo),
+      uses_shader_module_id(UsesShaderModuleId(*this)),
       stage_states(GetStageStates(*state_data, *this, csm_states)),
       create_info_shaders(GetCreateInfoShaders(*this)),
       active_shaders(create_info_shaders),  // RTX has no linking shaders
       active_slots(GetActiveSlots(stage_states)),
       max_active_slot(GetMaxActiveSlot(active_slots)),
-      uses_shader_module_id(UsesShaderModuleId(*this)),
       descriptor_buffer_mode((create_info.graphics.flags & VK_PIPELINE_CREATE_DESCRIPTOR_BUFFER_BIT_EXT) != 0),
       uses_pipeline_robustness(UsesPipelineRobustness(PNext(), *this)),
       csm_states(csm_states),
