@@ -18,7 +18,7 @@
  */
 
 #include "state_tracker/render_pass_state.h"
-#include "convert_to_renderpass2.h"
+#include "utils/convert_to_renderpass2.h"
 #include "state_tracker/image_state.h"
 
 static const VkImageLayout kInvalidLayout = VK_IMAGE_LAYOUT_MAX_ENUM;
