@@ -31,19 +31,21 @@
 
 class PIPELINE_STATE;
 
+// This is the common info for both OpDecorate and OpMemberDecorate
 // Used to keep track of all decorations applied to any instruction
-struct DecorationSet {
+struct DecorationBase {
     enum FlagBit {
         patch_bit = 1 << 0,
         block_bit = 1 << 1,
         buffer_block_bit = 1 << 2,
         nonwritable_bit = 1 << 3,
-        builtin_bit = 1 << 4,
-        nonreadable_bit = 1 << 5,
-        per_vertex_bit = 1 << 6,
-        passthrough_bit = 1 << 7,
-        aliased_bit = 1 << 8,
-        input_attachment_bit = 1 << 9,
+        nonreadable_bit = 1 << 4,
+        per_vertex_bit = 1 << 5,
+        passthrough_bit = 1 << 6,
+        aliased_bit = 1 << 7,
+        input_attachment_bit = 1 << 8,
+        per_task_nv = 1 << 9,
+        per_primitive_ext = 1 << 10,
     };
     static constexpr uint32_t kInvalidValue = std::numeric_limits<uint32_t>::max();
 
@@ -52,38 +54,170 @@ struct DecorationSet {
 
     // When being used as an User-defined Variable (input, output, rtx)
     uint32_t location = kInvalidValue;
+    // Component is optional and spec says it is 0 if not defined
     uint32_t component = 0;
 
-    // For descriptors
-    uint32_t set = 0;
-    uint32_t binding = 0;
-
+    // A given object can only have a single BuiltIn OpDecoration
     uint32_t builtin = kInvalidValue;
 
-    // Value of InputAttachmentIndex the variable starts
-    uint32_t input_attachment_index_start = kInvalidValue;
-
     void Add(uint32_t decoration, uint32_t value);
     bool Has(FlagBit flag_bit) const { return (flags & flag_bit) != 0; }
 };
 
-// vkspec.html#interfaces-iointerfaces-user describes 'User-defined Variable Interface'
+// subset only for OpDecorate
+// Can't have nested structs with OpMemberDecorate, this class prevents accidently creating a 2nd level of member decorations,
+struct DecorationSet : public DecorationBase {
+    // For descriptors
+    uint32_t set = 0;
+    uint32_t binding = 0;
+
+    // Value of InputAttachmentIndex the variable starts
+    uint32_t input_attachment_index_start = kInvalidValue;
+
+    // <index into struct, DecorationBase>
+    vvl::unordered_map<uint32_t, DecorationBase> member_decorations;
+
+    void Add(uint32_t decoration, uint32_t value);
+    bool HasBuiltIn() const;
+    bool AllMemberHave(FlagBit flag_bit) const;
+};
+
+// Tracking of OpExecutionMode / OpExecutionModeId values
+struct ExecutionModeSet {
+    enum FlagBit {
+        output_points_bit = 1 << 0,
+        point_mode_bit = 1 << 1,
+        post_depth_coverage_bit = 1 << 2,
+        local_size_bit = 1 << 3,
+        local_size_id_bit = 1 << 4,
+        iso_lines_bit = 1 << 5,
+        xfb_bit = 1 << 6,
+        early_fragment_test_bit = 1 << 7,
+        subgroup_uniform_control_flow_bit = 1 << 8,
+
+        signed_zero_inf_nan_preserve_width_16 = 1 << 9,
+        signed_zero_inf_nan_preserve_width_32 = 1 << 10,
+        signed_zero_inf_nan_preserve_width_64 = 1 << 11,
+        denorm_preserve_width_16 = 1 << 12,
+        denorm_preserve_width_32 = 1 << 13,
+        denorm_preserve_width_64 = 1 << 14,
+        denorm_flush_to_zero_width_16 = 1 << 15,
+        denorm_flush_to_zero_width_32 = 1 << 16,
+        denorm_flush_to_zero_width_64 = 1 << 17,
+        rounding_mode_rte_width_16 = 1 << 18,
+        rounding_mode_rte_width_32 = 1 << 19,
+        rounding_mode_rte_width_64 = 1 << 20,
+        rounding_mode_rtz_width_16 = 1 << 21,
+        rounding_mode_rtz_width_32 = 1 << 22,
+        rounding_mode_rtz_width_64 = 1 << 23,
+    };
+    static constexpr uint32_t kInvalidValue = std::numeric_limits<uint32_t>::max();
+
+    // bits to know if things have been set or not by a Decoration
+    uint32_t flags = 0;
+
+    VkPrimitiveTopology primitive_topology = VK_PRIMITIVE_TOPOLOGY_MAX_ENUM;
+
+    // SPIR-V spec says only LocalSize or LocalSizeId can be used, so can share
+    uint32_t local_size_x = kInvalidValue;
+    uint32_t local_size_y = kInvalidValue;
+    uint32_t local_size_z = kInvalidValue;
+
+    uint32_t output_vertices = 0;
+    uint32_t output_primitives = 0;
+    uint32_t invocations = 0;
+
+    void Add(const Instruction &insn);
+    bool Has(FlagBit flag_bit) const { return (flags & flag_bit) != 0; }
+};
+
+// Contains all the details for a OpTypeStruct
+struct TypeStructInfo {
+    const uint32_t id;
+    const uint32_t length;  // number of elements
+    const DecorationSet &decorations;
+    std::vector<uint32_t> member_ids;
+
+    TypeStructInfo(const SHADER_MODULE_STATE &module_state, const Instruction &struct_insn);
+};
+
+// A slot is a <Location, Component> mapping
+struct InterfaceSlot {
+    // A Location is made up of 4 Components
+    // Example: Location 2, Component 1
+    // L0 : [ C0, C1, C2, C3 ]
+    // L1 : [ C0, C1, C2, C3 ]
+    // L2 : [ C0, C1, C2, C3 ]
+    //            ^
+    // index == 9 == (Location * 4) + Component
+    const uint32_t slot = 0;  // default
+
+    // Information about the variable type
+    // Easier to find this information once then re-look each time (mainly for structs)
+    const uint32_t type = 0;  // Opcode of OpType*
+    const uint32_t bit_width = 0;
+
+    uint32_t Location() const { return slot / 4; }
+    uint32_t Component() const { return slot % 4; }
+    std::string Describe() const;
+    // Having a single uint32_t slot allows a 64-bit Vec3 to pass in (Loc 0, Comp 5) and have it automatically mean (Loc 1, Comp 1)
+    InterfaceSlot(uint32_t location, uint32_t component, uint32_t type, uint32_t bit_width)
+        : slot(GetSlotValue(location, component)), type(type), bit_width(bit_width) {}
+    InterfaceSlot(uint32_t slot, uint32_t type, uint32_t bit_width) : slot(slot), type(type), bit_width(bit_width) {}
+
+    bool operator<(const InterfaceSlot &rhs) const { return slot < rhs.slot; }
+    bool operator==(const InterfaceSlot &rhs) const { return slot == rhs.slot; }
+    struct Hash {
+        std::size_t operator()(const InterfaceSlot &object) const { return object.slot; }
+    };
+
+    uint32_t GetSlotValue(uint32_t location, uint32_t component) { return (location * 4) + component; }
+};
+
+// Common info needed for all OpVariable
+struct VariableBase {
+    const uint32_t id;
+    const uint32_t type_id;
+    const spv::StorageClass storage_class;
+    const DecorationSet &decorations;
+    std::shared_ptr<const TypeStructInfo> type_struct_info;  // null if no struct type
+    // If variable is accessed from mulitple entrypoint, create a seperate VariableBase object as info about it being access will be
+    // different
+    const VkShaderStageFlagBits stage;
+    VariableBase(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
+};
+
 // These are Input/Output OpVariable that go in-between stages
 // (also for example the input to a Vertex and output of the Fragment).
 // These are always ints/floats (not images or samplers).
 // Besides the input vertex binding, all of these are fully known at pipeline creation time
-struct UserDefinedInterfaceVariable {
-    uint32_t id;
-    uint32_t type_id;
+//
+// These include both BuiltIns and User Defined, while there are difference in member variables, the variables are needed for the
+// common logic so its easier using the same object in the end
+struct StageInteraceVariable : public VariableBase {
+    // Only will be true in BuiltIns
+    const bool is_patch;
+    const bool is_per_vertex;   // VK_KHR_fragment_shader_barycentric
+    const bool is_per_task_nv;  // VK_NV_mesh_shader
 
-    // if a block with multiple location, track which offset to only check the first
-    uint32_t offset;
+    const bool is_array_interface;
+    const Instruction &base_type;
+    const bool is_builtin;
+    bool nested_struct;
 
-    bool is_patch{false};
+    const std::vector<InterfaceSlot> interface_slots;  // Only for User Defined variables
+    const std::vector<uint32_t> builtin_block;
+    uint32_t total_builtin_components = 0;
 
-    UserDefinedInterfaceVariable() : id(0), type_id(0), offset(0) {}
+    StageInteraceVariable(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
 
-    UserDefinedInterfaceVariable(const Instruction *insn) : id(insn->Word(2)), type_id(insn->Word(1)), offset(0) {}
+  protected:
+    static bool IsArrayInterface(const StageInteraceVariable &variable);
+    static const Instruction &FindBaseType(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
+    static bool IsBuiltin(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
+    static std::vector<InterfaceSlot> GetInterfaceSlots(StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
+    static std::vector<uint32_t> GetBuiltinBlock(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
+    static uint32_t GetBuiltinComponents(const StageInteraceVariable &variable, const SHADER_MODULE_STATE &module_state);
 };
 
 // vkspec.html#interfaces-resources describes 'Shader Resource Interface'
@@ -92,14 +226,7 @@ struct UserDefinedInterfaceVariable {
 // not known until the descriptors are bound.
 // The main purpose of this struct is to track what operations are statically done so
 // at draw/submit time we can cross reference with the last bound descriptor.
-struct ResourceInterfaceVariable {
-    uint32_t id;
-    uint32_t type_id;
-    spv::StorageClass storage_class;
-
-    VkShaderStageFlagBits stage;
-    DecorationSet decorations;
-
+struct ResourceInterfaceVariable : public VariableBase {
     // If the type is a OpTypeArray save the length
     uint32_t array_length = 0;
     bool runtime_array = false;  // OpTypeRuntimeArray - can't validate length until run time
@@ -139,20 +266,15 @@ struct ResourceInterfaceVariable {
     bool is_write_without_format{false};  // For storage images
     bool is_dref_operation{false};
 
-    ResourceInterfaceVariable(const SHADER_MODULE_STATE &module_state, const Instruction *insn, VkShaderStageFlagBits stage);
+    ResourceInterfaceVariable(const SHADER_MODULE_STATE &module_state, const Instruction &insn, VkShaderStageFlagBits stage);
 };
 
-std::vector<uint32_t> FindEntrypointInterfaces(const Instruction &entrypoint);
-
 enum FORMAT_TYPE {
     FORMAT_TYPE_FLOAT = 1,  // UNORM, SNORM, FLOAT, USCALED, SSCALED, SRGB -- anything we consider float in the shader
     FORMAT_TYPE_SINT = 2,
     FORMAT_TYPE_UINT = 4,
 };
 
-// <Location, Component>
-typedef std::pair<uint32_t, uint32_t> location_t;
-
 struct SHADER_MODULE_STATE : public BASE_NODE {
     // Contains all the details for a OpTypeStruct
     struct StructInfo {
@@ -217,16 +339,46 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
         // There is no single unique item for a single entry point
         const Instruction &entrypoint_insn;  // OpEntryPoint instruction
         const VkShaderStageFlagBits stage;
+        const uint32_t id;
         const std::string name;
         // All ids that can be accessed from the entry point
         vvl::unordered_set<uint32_t> accessible_ids;
 
-        std::vector<UserDefinedInterfaceVariable> user_defined_interface_variables;
+        const ExecutionModeSet &execution_mode;
+
         std::vector<ResourceInterfaceVariable> resource_interface_variables;
+        std::vector<StageInteraceVariable> stage_interface_variables;
+        // Easier to lookup without having to check for the is_builtin bool
+        // "Built-in interface variables" - vkspec.html#interfaces-iointerfaces-builtin
+        std::vector<const StageInteraceVariable *> built_in_variables;
+        // "User-defined Variable Interface" - vkspec.html#interfaces-iointerfaces-user
+        std::vector<const StageInteraceVariable *> user_defined_interface_variables;
+
+        // Lookup map from Interface slot to the variable in that spot
+        // spirv-val guarantees no overlap so 2 variables won't have same slot
+        std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> input_interface_slots;
+        std::unordered_map<InterfaceSlot, const StageInteraceVariable *, InterfaceSlot::Hash> output_interface_slots;
+        // Uesd for limit check
+        const StageInteraceVariable *max_input_slot_variable = nullptr;
+        const StageInteraceVariable *max_output_slot_variable = nullptr;
+        const InterfaceSlot *max_input_slot = nullptr;
+        const InterfaceSlot *max_output_slot = nullptr;
+        uint32_t builtin_input_components = 0;
+        uint32_t builtin_output_components = 0;
 
         StructInfo push_constant_used_in_shader;
 
-        EntryPoint(const SHADER_MODULE_STATE &module_state, const Instruction &entrypoint);
+        bool emit_vertex_geometry{false};
+
+        // Mark if a BuiltIn is written to
+        bool written_builtin_point_size{false};
+        bool written_builtin_primitive_shading_rate_khr{false};
+        bool written_builtin_viewport_index{false};
+        bool written_builtin_viewport_mask_nv{false};
+
+        bool has_passthrough{false};
+
+        EntryPoint(const SHADER_MODULE_STATE &module_state, const Instruction &entrypoint_insn);
     };
 
     // Static/const data extracted from a SPIRV module at initialization time
@@ -245,6 +397,13 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
         vvl::unordered_map<uint32_t, const Instruction *> definitions;
 
         vvl::unordered_map<uint32_t, DecorationSet> decorations;
+        DecorationSet empty_decoration;  // all zero values, allows use to return a reference and not a copy each time
+
+        // Execution Modes are tied to a Function <id>, multiple EntryPoints can point to the same Funciton <id>
+        // Keep a mapping so each EntryPoint can grab a reference to it
+        vvl::unordered_map<uint32_t, ExecutionModeSet> execution_modes;
+        ExecutionModeSet empty_execution_mode;  // all zero values, allows use to return a reference and not a copy each time
+
         // <Specialization constant ID -> target ID> mapping
         vvl::unordered_map<uint32_t, uint32_t> spec_const_map;
         // Find all decoration instructions to prevent relooping module later - many checks need this info
@@ -252,18 +411,28 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
         std::vector<const Instruction *> member_decoration_inst;
         // Find all variable instructions to prevent relookping module later
         std::vector<const Instruction *> variable_inst;
-        // Execution are not tied to an entry point and are their own mapping tied to entry point function
-        // [OpEntryPoint function <id> operand] : [Execution Mode Instruction list]
-        vvl::unordered_map<uint32_t, std::vector<const Instruction *>> execution_mode_inst;
         // both OpDecorate and OpMemberDecorate builtin instructions
         std::vector<const Instruction *> builtin_decoration_inst;
+        // OpEmitStreamVertex/OpEndStreamPrimitive
+        std::vector<const Instruction *> transform_feedback_stream_inst;
+        // BuiltIn we just care about existing or not, don't have to be written to
+        bool has_builtin_layer{false};
+        bool has_builtin_fully_covered{false};
+        bool has_builtin_workgroup_size{false};
+        uint32_t builtin_workgroup_size_id = 0;
+
         std::vector<const Instruction *> atomic_inst;
         std::vector<spv::Capability> capability_list;
 
         bool has_specialization_constants{false};
         bool has_invocation_repack_instruction{false};
 
-        std::vector<EntryPoint> entry_points;
+        // EntryPoint has pointer references inside it that need to be preserved
+        std::vector<std::shared_ptr<EntryPoint>> entry_points;
+
+        std::vector<std::shared_ptr<TypeStructInfo>> type_structs;  // All OpTypeStruct objects
+        // <OpTypeStruct ID, info> - used for faster lookup as there can many structs
+        vvl::unordered_map<uint32_t, std::shared_ptr<const TypeStructInfo>> type_struct_map;
 
         bool has_group_decoration{false};
 
@@ -293,22 +462,17 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
 
     uint32_t gpu_validation_shader_id{std::numeric_limits<uint32_t>::max()};
 
-    explicit SHADER_MODULE_STATE(vvl::span<const uint32_t> code, spv_target_env env = SPV_ENV_VULKAN_1_0)
+    explicit SHADER_MODULE_STATE(vvl::span<const uint32_t> code)
         : BASE_NODE(static_cast<VkShaderModule>(VK_NULL_HANDLE), kVulkanObjectTypeShaderModule),
           words_(code.begin(), code.end()),
-          static_data_(*this) {
-        PreprocessShaderBinary(env);
-    }
+          static_data_(*this) {}
 
-    SHADER_MODULE_STATE(const VkShaderModuleCreateInfo &create_info, VkShaderModule shaderModule, spv_target_env env,
-                        uint32_t unique_shader_id)
+    SHADER_MODULE_STATE(const VkShaderModuleCreateInfo &create_info, VkShaderModule shaderModule, uint32_t unique_shader_id)
         : BASE_NODE(shaderModule, kVulkanObjectTypeShaderModule),
           words_(create_info.pCode, create_info.pCode + create_info.codeSize / sizeof(uint32_t)),
           has_valid_spirv(true),
           static_data_(*this),
-          gpu_validation_shader_id(unique_shader_id) {
-        PreprocessShaderBinary(env);
-    }
+          gpu_validation_shader_id(unique_shader_id) {}
 
     SHADER_MODULE_STATE() : BASE_NODE(static_cast<VkShaderModule>(VK_NULL_HANDLE), kVulkanObjectTypeShaderModule) {}
 
@@ -319,75 +483,72 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
     }
 
     const std::vector<Instruction> &GetInstructions() const { return static_data_.instructions; }
-    const std::vector<const Instruction *> &GetDecorationInstructions() const { return static_data_.decoration_inst; }
-    const std::vector<const Instruction *> &GetMemberDecorationInstructions() const { return static_data_.member_decoration_inst; }
-    const std::vector<const Instruction *> &GetAtomicInstructions() const { return static_data_.atomic_inst; }
-    const std::vector<const Instruction *> &GetVariableInstructions() const { return static_data_.variable_inst; }
-    const std::vector<ResourceInterfaceVariable> *GetResourceInterfaceVariable(const Instruction &entrypoint) const {
-        for (const auto &entry_point : static_data_.entry_points) {
-            if (entry_point.entrypoint_insn == entrypoint) {
-                return &entry_point.resource_interface_variables;
-            }
-        }
-        return nullptr;
-    }
 
     const std::vector<const Instruction *> FindVariableAccesses(uint32_t variable_id, const std::vector<uint32_t> &access_ids,
                                                                 bool atomic) const;
 
-    const vvl::unordered_map<uint32_t, std::vector<const Instruction *>> &GetExecutionModeInstructions() const {
-        return static_data_.execution_mode_inst;
-    }
-
-    const std::vector<const Instruction *> &GetBuiltinDecorationList() const { return static_data_.builtin_decoration_inst; }
-
-    const vvl::unordered_map<uint32_t, uint32_t> &GetSpecConstMap() const { return static_data_.spec_const_map; }
-
-    bool HasSpecConstants() const { return static_data_.has_specialization_constants; }
-    bool HasInvocationRepackInstruction() const { return static_data_.has_invocation_repack_instruction; }
-
     bool HasMultipleEntryPoints() const { return static_data_.entry_points.size() > 1; }
 
     VkShaderModule vk_shader_module() const { return handle_.Cast<VkShaderModule>(); }
 
-    DecorationSet GetDecorationSet(uint32_t id) const {
-        // return the actual decorations for this id, or a default set.
-        auto it = static_data_.decorations.find(id);
-        if (it != static_data_.decorations.end()) return it->second;
-        return DecorationSet();
+    const DecorationSet &GetDecorationSet(uint32_t id) const {
+        // return the actual decorations for this id, or a default empty set.
+        const auto it = static_data_.decorations.find(id);
+        return (it != static_data_.decorations.end()) ? it->second : static_data_.empty_decoration;
+    }
+
+    const ExecutionModeSet &GetExecutionModeSet(uint32_t function_id) const {
+        // return the actual execution modes for this id, or a default empty set.
+        const auto it = static_data_.execution_modes.find(function_id);
+        return (it != static_data_.execution_modes.end()) ? it->second : static_data_.empty_execution_mode;
+    }
+
+    std::shared_ptr<const TypeStructInfo> GetTypeStructInfo(uint32_t struct_id) const {
+        // return the actual execution modes for this id, or a default empty set.
+        const auto it = static_data_.type_struct_map.find(struct_id);
+        return (it != static_data_.type_struct_map.end()) ? it->second : nullptr;
+    }
+    // Overload to walk down and find the OpTypeStruct
+    std::shared_ptr<const TypeStructInfo> GetTypeStructInfo(const Instruction *insn) const {
+        while (true) {
+            if (insn->Opcode() == spv::OpVariable) {
+                insn = FindDef(insn->Word(1));
+            } else if (insn->Opcode() == spv::OpTypePointer) {
+                insn = FindDef(insn->Word(3));
+            } else if (insn->Opcode() == spv::OpTypeArray || insn->Opcode() == spv::OpTypeRuntimeArray) {
+                insn = FindDef(insn->Word(2));
+            } else if (insn->Opcode() == spv::OpTypeStruct) {
+                return GetTypeStructInfo(insn->Word(1));
+            } else {
+                return nullptr;
+            }
+        }
     }
 
     // Used to get human readable strings for error messages
     void DescribeTypeInner(std::ostringstream &ss, uint32_t type, uint32_t indent) const;
     std::string DescribeType(uint32_t type) const;
 
-    std::optional<VkPrimitiveTopology> GetTopology(const Instruction &entrypoint) const;
+    std::optional<VkPrimitiveTopology> GetTopology(const EntryPoint &entrypoint) const;
 
     const StructInfo *FindEntrypointPushConstant(char const *name, VkShaderStageFlagBits stageBits) const;
-    std::optional<Instruction> FindEntrypoint(char const *name, VkShaderStageFlagBits stageBits) const;
-    bool FindLocalSize(const Instruction &entrypoint, uint32_t &local_size_x, uint32_t &local_size_y, uint32_t &local_size_z) const;
+    std::shared_ptr<const EntryPoint> FindEntrypoint(char const *name, VkShaderStageFlagBits stageBits) const;
+    [[nodiscard]] bool FindLocalSize(const EntryPoint &entrypoint, uint32_t &local_size_x, uint32_t &local_size_y,
+                                     uint32_t &local_size_z) const;
 
     const Instruction *GetConstantDef(uint32_t id) const;
     uint32_t GetConstantValueById(uint32_t id) const;
     spv::Dim GetShaderResourceDimensionality(const ResourceInterfaceVariable &resource) const;
-    uint32_t GetLocationsConsumedByType(uint32_t type, bool strip_array_level) const;
-    uint32_t GetComponentsConsumedByType(uint32_t type, bool strip_array_level) const;
+    uint32_t GetLocationsConsumedByType(uint32_t type) const;
+    uint32_t GetComponentsConsumedByType(uint32_t type) const;
     uint32_t GetFundamentalType(uint32_t type) const;
-    const Instruction *GetStructType(const Instruction *insn, bool is_array_of_verts) const;
+    const Instruction *GetStructType(const Instruction *insn) const;
 
     uint32_t DescriptorTypeToReqs(uint32_t type_id) const;
 
-    bool IsBuiltInWritten(const Instruction *builtin_insn, const Instruction &entrypoint) const;
-
-    // State tracking helpers for collecting interface information
-    vvl::unordered_set<uint32_t> CollectWritableOutputLocationinFS(const Instruction &entrypoint) const;
-    bool CollectInterfaceBlockMembers(std::map<location_t, UserDefinedInterfaceVariable> *out, bool is_array_of_verts,
-                                      bool is_patch, const Instruction *variable_insn) const;
-    std::map<location_t, UserDefinedInterfaceVariable> CollectInterfaceByLocation(const Instruction &entrypoint,
-                                                                                  spv::StorageClass sinterface,
-                                                                                  bool is_array_of_verts) const;
-    std::vector<uint32_t> CollectBuiltinBlockMembers(const Instruction &entrypoint, uint32_t storageClass) const;
+    bool IsBuiltInWritten(const Instruction *builtin_insn, const EntryPoint &entrypoint) const;
 
+    // Instruction helpers that need the knowledge of the whole SPIR-V module
     uint32_t GetNumComponentsInBaseType(const Instruction *insn) const;
     uint32_t GetTypeBitsSize(const Instruction *insn) const;
     uint32_t GetTypeBytesSize(const Instruction *insn) const;
@@ -396,11 +557,6 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
     uint32_t GetTypeId(uint32_t id) const;
     uint32_t GetTexelComponentCount(const Instruction &insn) const;
 
-    bool WritesToGlLayer() const {
-        return std::any_of(static_data_.builtin_decoration_inst.begin(), static_data_.builtin_decoration_inst.end(),
-                           [](const Instruction *insn) { return insn->GetBuiltIn() == spv::BuiltInLayer; });
-    }
-
     bool HasCapability(spv::Capability find_capability) const {
         return std::any_of(static_data_.capability_list.begin(), static_data_.capability_list.end(),
                            [find_capability](const spv::Capability &capability) { return capability == find_capability; });
@@ -408,19 +564,14 @@ struct SHADER_MODULE_STATE : public BASE_NODE {
 
     // Used to set push constants values at shader module initialization time
     static void SetPushConstantUsedInShader(const SHADER_MODULE_STATE &module_state,
-                                            std::vector<SHADER_MODULE_STATE::EntryPoint> &entry_points);
+                                            std::vector<std::shared_ptr<SHADER_MODULE_STATE::EntryPoint>> &entry_points);
 
   private:
-    // Functions used for initialization only
-    // Used to populate the shader module object
-    void PreprocessShaderBinary(spv_target_env env);
-
     // The following are all helper functions to set the push constants values by tracking if the values are accessed in the entry
     // point functions and which offset in the structs are used
     uint32_t UpdateOffset(uint32_t offset, const std::vector<uint32_t> &array_indices, const StructInfo &data) const;
     void SetUsedBytes(uint32_t offset, const std::vector<uint32_t> &array_indices, const StructInfo &data) const;
-    void DefineStructMember(const Instruction *insn, std::vector<const Instruction *> &member_decorate_insn,
-                            StructInfo &data) const;
+    void DefineStructMember(const Instruction *insn, StructInfo &data) const;
     void RunUsedArray(uint32_t offset, std::vector<uint32_t> array_indices, uint32_t access_chain_word_index,
                       const Instruction *access_chain, const StructInfo &data) const;
     void RunUsedStruct(uint32_t offset, uint32_t access_chain_word_index, const Instruction *access_chain,
