@@ -65,11 +65,27 @@ def RunShellCmd(command, start_dir = PROJECT_ROOT, env=None, verbose=False):
 def IsWindows(): return 'windows' == platform.system().lower()
 
 #
-# Verify consistency of generated source code
-def CheckVVLCodegenConsistency(config):
-    print("Check Generated Source Code Consistency")
+# Run VVL scripts
+def CheckVVL(config):
     ext_dir = externalDir(config)
-    gen_check_cmd = f'python3 scripts/generate_source.py --verify {ext_dir}/Vulkan-Headers/registry {ext_dir}/SPIRV-Headers/include/spirv/unified1/'
+    vulkan_registry = ext_dir + "/Vulkan-Headers/registry"
+    spirv_unified = ext_dir + "/SPIRV-Headers/include/spirv/unified1/"
+
+    # Scripts depend on modules within the Vulkan registry.
+    if "PYTHONPATH" in os.environ:
+        print(f"Using provided PYTHONPATH {os.environ['PYTHONPATH']}")
+    else:
+        os.environ['PYTHONPATH'] = vulkan_registry
+
+    # Verify consistency of generated source code
+    print("Check Generated Source Code Consistency")
+    gen_check_cmd = f'python3 scripts/generate_source.py --verify {vulkan_registry} {spirv_unified}'
+    RunShellCmd(gen_check_cmd)
+
+    print('Run vk_validation_stats.py')
+    valid_usage_json = vulkan_registry + "/validusage.json"
+    text_file = RepoRelative(f'{VVL_BUILD_DIR}/layers/vuid_coverage_database.txt')
+    gen_check_cmd = f'python3 scripts/vk_validation_stats.py {valid_usage_json} -text {text_file}'
     RunShellCmd(gen_check_cmd)
 
 #
@@ -98,13 +114,6 @@ def BuildVVL(config, cmake_args, build_tests):
     install_cmd = f'cmake --install {VVL_BUILD_DIR} --prefix {TEST_INSTALL_DIR}'
     RunShellCmd(install_cmd)
 
-    print('Run vk_validation_stats.py')
-    ext_dir = externalDir(config)
-    stats_script = RepoRelative('scripts/vk_validation_stats.py')
-    validusage = os.path.join(ext_dir, 'Vulkan-Headers', 'registry', 'validusage.json')
-    outfile = os.path.join('layers', 'vuid_coverage_database.txt')
-    RunShellCmd(f'{sys.executable} {stats_script} {validusage} -text {outfile}', VVL_BUILD_DIR)
-
 #
 # Prepare Loader for executing Layer Validation Tests
 def BuildLoader():
@@ -192,7 +201,7 @@ def BuildProfileLayer():
 
 #
 # Run the Layer Validation Tests
-def RunVVLTests(config):
+def RunVVLTests():
     print("Run Vulkan-ValidationLayer Tests using Mock ICD")
 
     if IsWindows():
@@ -211,12 +220,21 @@ def RunVVLTests(config):
 
     lvt_env['VK_INSTANCE_LAYERS'] = 'VK_LAYER_KHRONOS_validation' + os.pathsep + 'VK_LAYER_KHRONOS_profiles'
     lvt_env['VK_KHRONOS_PROFILES_SIMULATE_CAPABILITIES'] = 'SIMULATE_API_VERSION_BIT,SIMULATE_FEATURES_BIT,SIMULATE_PROPERTIES_BIT,SIMULATE_EXTENSIONS_BIT,SIMULATE_FORMATS_BIT,SIMULATE_QUEUE_FAMILY_PROPERTIES_BIT'
-    lvt_env['VK_KHRONOS_PROFILES_EMULATE_PORTABILITY'] = 'false'
+
+    # By default use the max_profile.json
+    if "VK_KHRONOS_PROFILES_PROFILE_FILE" not in os.environ:
+        lvt_env['VK_KHRONOS_PROFILES_PROFILE_FILE'] = RepoRelative('tests/device_profiles/max_profile.json')
+
+    # By default set portability to false
+    if "VK_KHRONOS_PROFILES_EMULATE_PORTABILITY" not in os.environ:
+        lvt_env['VK_KHRONOS_PROFILES_EMULATE_PORTABILITY'] = 'false'
+
     lvt_env['VK_KHRONOS_PROFILES_DEBUG_REPORTS'] = 'DEBUG_REPORT_ERROR_BIT'
 
     RunShellCmd(lvt_cmd, env=lvt_env)
-    print("Re-Running multithreaded tests with VK_LAYER_FINE_GRAINED_LOCKING=1:")
-    lvt_env['VK_LAYER_FINE_GRAINED_LOCKING'] = '1'
+
+    print("Re-Running multithreaded tests with VK_LAYER_FINE_GRAINED_LOCKING disabled")
+    lvt_env['VK_LAYER_FINE_GRAINED_LOCKING'] = '0'
     RunShellCmd(lvt_cmd + ' --gtest_filter=*Thread*', env=lvt_env)
 
 def GetArgParser():
