@@ -106,7 +106,7 @@ def main(argv):
     # get directory where generators will run
     if args.verify or args.incremental:
         # generate in temp directory so we can compare or copy later
-        temp_obj = tempfile.TemporaryDirectory(prefix='VulkanVL_generated_source_')
+        temp_obj = tempfile.TemporaryDirectory(prefix='vvl_codegen_')
         temp_dir = temp_obj.name
         gen_dir = temp_dir
     else:
