@@ -33,7 +33,7 @@ def main():
 
     try:
         common_ci.BuildVVL(config = config, cmake_args = args.cmake, build_tests = "OFF")
-        common_ci.CheckVVLCodegenConsistency(config = config)
+        common_ci.CheckVVL(config = config)
 
     except subprocess.CalledProcessError as proc_error:
         print('Command "%s" failed with return code %s' % (' '.join(proc_error.cmd), proc_error.returncode))
