@@ -1,6 +1,6 @@
 #!/usr/bin/python3
 #
-# Copyright (c) 2013-2019 The Khronos Group Inc.
+# Copyright (c) 2013-2023 The Khronos Group Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -351,36 +351,6 @@ def makeGenOpts(args):
             helper_file_type  = 'typemap_helper_header')
         ]
 
-    # Helper file generator options for corechecks_optick_instrumentation.h
-    genOpts['corechecks_optick_instrumentation.h'] = [
-          HelperFileOutputGenerator,
-          HelperFileOutputGeneratorOptions(
-            conventions       = conventions,
-            filename          = 'corechecks_optick_instrumentation.h',
-            directory         = directory,
-            versions          = featuresPat,
-            emitversions      = featuresPat,
-            addExtensions     = addExtensionsPat,
-            removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'optick_instrumentation_header')
-        ]
-
-    # Helper file generator options for corechecks_optick_instrumentation.cpp
-    genOpts['corechecks_optick_instrumentation.cpp'] = [
-          HelperFileOutputGenerator,
-          HelperFileOutputGeneratorOptions(
-            conventions       = conventions,
-            filename          = 'corechecks_optick_instrumentation.cpp',
-            directory         = directory,
-            versions          = featuresPat,
-            emitversions      = featuresPat,
-            addExtensions     = addExtensionsPat,
-            removeExtensions  = removeExtensionsPat,
-            emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'optick_instrumentation_source')
-        ]
-
     # Layer chassis related generation structs
     # Options for layer chassis header
     genOpts['chassis.h'] = [
@@ -490,34 +460,34 @@ def makeGenOpts(args):
 
 # Create an API generator and corresponding generator options based on
 # the requested target and command line options.
-    # Helper file generator options for synchronization_validation_types.h
-    genOpts['synchronization_validation_types.h'] = [
+    # Helper file generator options for sync_validation_types.h
+    genOpts['sync_validation_types.h'] = [
           HelperFileOutputGenerator,
           HelperFileOutputGeneratorOptions(
             conventions       = conventions,
-            filename          = 'synchronization_validation_types.h',
+            filename          = 'sync_validation_types.h',
             directory         = directory,
             versions          = featuresPat,
             emitversions      = featuresPat,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'synchronization_helper_header',
+            helper_file_type  = 'sync_helper_header',
             valid_usage_path  = args.scripts)
         ]
 
-    genOpts['synchronization_validation_types.cpp'] = [
+    genOpts['sync_validation_types.cpp'] = [
           HelperFileOutputGenerator,
           HelperFileOutputGeneratorOptions(
             conventions       = conventions,
-            filename          = 'synchronization_validation_types.cpp',
+            filename          = 'sync_validation_types.cpp',
             directory         = directory,
             versions          = featuresPat,
             emitversions      = featuresPat,
             addExtensions     = addExtensionsPat,
             removeExtensions  = removeExtensionsPat,
             emitExtensions    = emitExtensionsPat,
-            helper_file_type  = 'synchronization_helper_source',
+            helper_file_type  = 'sync_helper_source',
             valid_usage_path  = args.scripts)
         ]
 
@@ -782,6 +752,9 @@ if __name__ == '__main__':
     tree = etree.parse(args.registry)
     endTimer(args.time, '* Time to make ElementTree =')
 
+    # Filter out non-Vulkan extensions
+    [exts.remove(e) for exts in tree.findall('extensions') for e in exts.findall('extension') if (sup := e.get('supported')) is not None and options.apiname not in sup.split(',')]
+
     # Load the XML tree into the registry object
     startTimer(args.time)
     reg.loadElementTree(tree)
