@@ -94,10 +94,10 @@ class SpirvValidationHelperOutputGenerator(OutputGenerator):
         # These 2 arrays SHOULD be empty when possible and when the SPIR-V Headers are updated these
         # should be attempted to be cleared
         self.capabilityExcludeList = [
-            'TextureBlockMatchQCOM',
-            'TextureBoxFilterQCOM',
-            'TextureSampleWeightedQCOM',
             'ClusterCullingShadingHUAWEI',
+            'TileImageColorReadAccessEXT',
+            'TileImageDepthReadAccessEXT',
+            'TileImageStencilReadAccessEXT',
         ]
 
         # There are some enums that share the same value in the SPIR-V header.
@@ -143,6 +143,7 @@ class SpirvValidationHelperOutputGenerator(OutputGenerator):
             {'vulkan' : 'VkPhysicalDeviceShaderIntegerDotProductFeaturesKHR', 'layer' : 'shader_integer_dot_product_features'},
             {'vulkan' : 'VkPhysicalDeviceShaderSubgroupUniformControlFlowFeaturesKHR', 'layer' : 'shader_subgroup_uniform_control_flow_features'},
             {'vulkan' : 'VkPhysicalDeviceRayTracingMaintenance1FeaturesKHR', 'layer' : 'ray_tracing_maintenance1_features'},
+            {'vulkan' : 'VkPhysicalDeviceImageProcessingFeaturesQCOM', 'layer' : 'image_processing_features'},
             {'vulkan' : 'VkPhysicalDeviceShaderCoreBuiltinsFeaturesARM', 'layer' : 'shader_core_builtins_features'},
         ]
 
