@@ -55,7 +55,7 @@ layer_source_files = [common_codegen.repo_relative(path) for path in [
     'layers/core_checks/image_validation.cpp',
     'layers/core_checks/image_layout_validation.cpp',
     'layers/core_checks/pipeline_validation.cpp',
-    'layers/object_tracker_utils.cpp',
+    'layers/object_tracker/object_tracker_utils.cpp',
     'layers/core_checks/query_validation.cpp',
     'layers/core_checks/queue_validation.cpp',
     'layers/core_checks/ray_tracing_validation.cpp',
@@ -68,6 +68,7 @@ layer_source_files = [common_codegen.repo_relative(path) for path in [
     'layers/sync/sync_vuid_maps.cpp',
     'layers/core_checks/video_validation.cpp',
     'layers/core_checks/wsi_validation.cpp',
+    'layers/core_checks/ycbcr_validation.cpp',
     'layers/generated/parameter_validation.cpp',
     'layers/generated/object_tracker.cpp',
     'layers/generated/spirv_validation_helper.cpp',
@@ -79,8 +80,8 @@ test_source_files = glob.glob(os.path.join(common_codegen.repo_relative('tests')
 unassigned_vuid_files = [common_codegen.repo_relative(path) for path in [
     'layers/best_practices/best_practices_error_enums.h',
     'layers/stateless/stateless_validation.h',
-    'layers/core_validation_error_enums.h',
-    'layers/object_lifetime_validation.h'
+    'layers/error_message/validation_error_enums.h',
+    'layers/object_tracker/object_lifetime_validation.h'
 ]]
 
 # These files should not change unless event there is a major refactoring in SPIR-V Tools
@@ -286,7 +287,7 @@ class ValidationSource:
 
 # Class to parse the validation layer test source and store testnames
 class ValidationTests:
-    def __init__(self, test_file_list, test_group_name=['VkLayerTest', 'VkPositiveLayerTest', 'VkWsiEnabledLayerTest', 'VkBestPracticesLayerTest']):
+    def __init__(self, test_file_list, test_group_name=['VkLayerTest', 'VkPositiveLayerTest', 'VkBestPracticesLayerTest']):
         self.test_files = test_file_list
         self.test_trigger_txt_list = []
         for tg in test_group_name:
