@@ -24,7 +24,7 @@
 #include <memory>
 #include <vector>
 
-#include "lvt_function_pointers.h"
+#include "generated/lvt_function_pointers.h"
 #include "test_common.h"
 
 namespace vk_testing {
@@ -326,12 +326,12 @@ class DeviceMemory : public internal::NonDispHandle<VkDeviceMemory> {
     // vkUnmapMemory()
     void unmap() const;
 	const auto &get_memory_allocate_info() { return memory_allocate_info_; }
-	
-	static VkMemoryAllocateInfo get_resource_alloc_info(const vk_testing::Device &dev, const VkMemoryRequirements &reqs,
-                                                        VkMemoryPropertyFlags mem_props, void *alloc_info_pnext = nullptr);
-                                                        
-  private:
-    VkMemoryAllocateInfo memory_allocate_info_{};
+
+        static VkMemoryAllocateInfo get_resource_alloc_info(const vk_testing::Device &dev, const VkMemoryRequirements &reqs,
+                                                            VkMemoryPropertyFlags mem_props, void *alloc_info_pnext = nullptr);
+
+      private:
+        VkMemoryAllocateInfo memory_allocate_info_{};
 };
 
 class Fence : public internal::NonDispHandle<VkFence> {
@@ -733,32 +733,6 @@ class AccelerationStructure : public internal::NonDispHandle<VkAccelerationStruc
     uint64_t opaque_handle_;
 };
 
-class AccelerationStructureKHR : public internal::NonDispHandle<VkAccelerationStructureKHR> {
-  public:
-    explicit AccelerationStructureKHR(){};
-    explicit AccelerationStructureKHR(const Device &dev, const VkAccelerationStructureCreateInfoKHR &info,
-                                      bool init_memory = true) {
-        init(dev, info, init_memory);
-    }
-    ~AccelerationStructureKHR() noexcept;
-    void destroy() noexcept;
-
-    // vkCreateAccelerationStructureNV
-    void init(const Device &dev, const VkAccelerationStructureCreateInfoKHR &info, bool init_memory = true);
-    uint64_t opaque_handle() const { return opaque_handle_; }
-
-    const VkAccelerationStructureCreateInfoKHR &info() const { return info_; }
-
-    const VkDevice &dev() const { return device(); }
-
-    [[nodiscard]] vk_testing::Buffer create_scratch_buffer(const Device &device, const VkBufferCreateInfo *pCreateInfo = nullptr,
-                                                           bool buffer_device_address = false) const;
-
-  private:
-    VkAccelerationStructureCreateInfoKHR info_;
-    uint64_t opaque_handle_;
-};
-
 class ShaderModule : public internal::NonDispHandle<VkShaderModule> {
   public:
     ~ShaderModule() noexcept;
@@ -991,10 +965,10 @@ class Framebuffer : public internal::NonDispHandle<VkFramebuffer> {
 class SamplerYcbcrConversion : public internal::NonDispHandle<VkSamplerYcbcrConversion> {
   public:
     SamplerYcbcrConversion() = default;
-    SamplerYcbcrConversion(const Device &dev, VkFormat format, bool khr) : khr_(khr) {
+    SamplerYcbcrConversion(const Device &dev, VkFormat format, bool khr = false) : khr_(khr) {
         init(dev, DefaultConversionInfo(format), khr);
     }
-    SamplerYcbcrConversion(const Device &dev, const VkSamplerYcbcrConversionCreateInfo &info, bool khr) : khr_(khr) {
+    SamplerYcbcrConversion(const Device &dev, const VkSamplerYcbcrConversionCreateInfo &info, bool khr = false) : khr_(khr) {
         init(dev, info, khr);
     }
     ~SamplerYcbcrConversion() noexcept;
