@@ -28,11 +28,11 @@
 #include "icd-spv.h"
 #include "test_common.h"
 #include "vk_layer_config.h"
-#include "vk_layer_data.h"
-#include "vk_format_utils.h"
+#include "containers/custom_containers.h"
+#include "generated/vk_format_utils.h"
 #include "render.h"
-#include "vk_typemap_helper.h"
-#include "convert_to_renderpass2.h"
+#include "generated/vk_typemap_helper.h"
+#include "utils/convert_to_renderpass2.h"
 
 #include <algorithm>
 #include <cmath>
@@ -117,6 +117,10 @@ static char const bindStateGeomPointSizeShaderText[] = R"glsl(
     layout (points) in;
     layout (points) out;
     layout (max_vertices = 1) out;
+    in gl_PerVertex {
+        vec4 gl_Position;
+        float gl_PointSize;
+    };
     void main() {
        gl_Position = vec4(1);
        gl_PointSize = 1.0;
@@ -173,6 +177,14 @@ static char const bindStateFragSubpassLoadInputText[] = R"glsl(
     }
 )glsl";
 
+static char const bindStateFragColorOutputText[] = R"glsl(
+    #version 460
+    layout(location=0) out vec4 color;
+    void main() {
+        color = vec4(1.0f);
+    }
+)glsl";
+
 [[maybe_unused]] static const char *bindStateRTShaderText = R"glsl(
     #version 460
     #extension GL_EXT_ray_tracing : require // Requires SPIR-V 1.5 (Vulkan 1.2)
@@ -265,8 +277,7 @@ class VkLayerTest : public VkRenderFramework {
 
     void Init(VkPhysicalDeviceFeatures *features = nullptr, VkPhysicalDeviceFeatures2 *features2 = nullptr,
               const VkCommandPoolCreateFlags flags = 0, void *instance_pnext = nullptr);
-    enum class WsiPreference { Default, Wayland, X11, XCB };
-    void AddSurfaceExtension(const WsiPreference preference = WsiPreference::Default);
+    void AddSurfaceExtension();
     VkCommandBufferObj *CommandBuffer();
     void OOBRayTracingShadersTestBody(bool gpu_assisted);
 
@@ -309,7 +320,6 @@ class VkLayerTest : public VkRenderFramework {
   protected:
     uint32_t m_instance_api_version = 0;
     uint32_t m_target_api_version = 0;
-    bool m_enableWSI;
 
     void SetTargetApiVersion(uint32_t target_api_version);
     uint32_t DeviceValidationVersion() const;
@@ -364,12 +374,6 @@ class VkArmBestPracticesLayerTest : public VkBestPracticesLayerTest {
 };
 class VkNvidiaBestPracticesLayerTest : public VkBestPracticesLayerTest {};
 
-class VkWsiEnabledLayerTest : public VkLayerTest {
-  public:
-  protected:
-    VkWsiEnabledLayerTest() { m_enableWSI = true; }
-};
-
 class VkGpuAssistedLayerTest : public VkLayerTest {
   public:
     VkValidationFeaturesEXT GetValidationFeatures();
@@ -868,9 +872,16 @@ class ThreadTimeoutHelper {
 
     struct Guard {
         Guard(ThreadTimeoutHelper &timeout_helper) : timeout_helper_(timeout_helper) {}
+        Guard(const Guard &) = delete;
+        Guard &operator=(const Guard &) = delete;
+
         ~Guard() { timeout_helper_.OnThreadDone(); }
+
         ThreadTimeoutHelper &timeout_helper_;
     };
+    // Mandatory elision of copy/move operations guarantees the destructor is not called
+    // (even in the presence of copy/move constructor) and the object is constructed directly
+    // into the destination storage: https://en.cppreference.com/w/cpp/language/copy_elision
     Guard ThreadGuard() { return Guard(*this); }
 
   private:
