@@ -14,6 +14,8 @@
 
 #include "ray_tracing_objects.h"
 
+#include "utils/vk_layer_utils.h"
+
 namespace rt {
 namespace as {
 
@@ -35,7 +37,14 @@ GeometryKHR &GeometryKHR::SetType(Type type) {
             vk_obj_.geometry.aabbs.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_AABBS_DATA_KHR;
             vk_obj_.geometry.aabbs.pNext = nullptr;
             break;
+        case Type::Instance:
+            vk_obj_.geometryType = VK_GEOMETRY_TYPE_INSTANCES_KHR;
+            vk_obj_.geometry.instances.sType = VK_STRUCTURE_TYPE_ACCELERATION_STRUCTURE_GEOMETRY_INSTANCES_DATA_KHR;
+            vk_obj_.geometry.instances.pNext = nullptr;
+            break;
         case Type::_INTERNAL_UNSPECIFIED:
+            [[fallthrough]];
+        default:
             assert(false);
             break;
     }
@@ -55,29 +64,50 @@ GeometryKHR &GeometryKHR::SetStride(VkDeviceSize stride) {
         case Type::AABB:
             vk_obj_.geometry.aabbs.stride = stride;
             break;
+        case Type::Instance:
+            [[fallthrough]];
         case Type::_INTERNAL_UNSPECIFIED:
+            [[fallthrough]];
+        default:
             assert(false);
             break;
     }
     return *this;
 }
 
-GeometryKHR &GeometryKHR::SetTrianglesVertexBuffer(vk_testing::Buffer &&vertex_buffer, uint32_t max_vertex,
-                                                   VkFormat vertex_format /*= VK_FORMAT_R32G32B32_SFLOAT*/,
-                                                   VkDeviceSize stride /*= 3 * sizeof(float)*/) {
-    triangles_.vertex_buffer = std::move(vertex_buffer);
+GeometryKHR &GeometryKHR::SetTrianglesDeviceVertexBuffer(vk_testing::Buffer &&vertex_buffer, uint32_t max_vertex,
+                                                         VkFormat vertex_format /*= VK_FORMAT_R32G32B32_SFLOAT*/,
+                                                         VkDeviceSize stride /*= 3 * sizeof(float)*/) {
+    triangles_.device_vertex_buffer = std::move(vertex_buffer);
     vk_obj_.geometry.triangles.vertexFormat = vertex_format;
-    vk_obj_.geometry.triangles.vertexData.deviceAddress = triangles_.vertex_buffer.address(vk_api_version_);
+    vk_obj_.geometry.triangles.vertexData.deviceAddress = triangles_.device_vertex_buffer.address(vk_api_version_);
     vk_obj_.geometry.triangles.maxVertex = max_vertex;
     vk_obj_.geometry.triangles.vertexStride = stride;
     return *this;
 }
 
-GeometryKHR &GeometryKHR::SetTrianglesIndexBuffer(vk_testing::Buffer &&index_buffer,
-                                                  VkIndexType index_type /*= VK_INDEX_TYPE_UINT32*/) {
-    triangles_.index_buffer = std::move(index_buffer);
+GeometryKHR &GeometryKHR::SetTrianglesHostVertexBuffer(std::unique_ptr<float[]> &&vertex_buffer, uint32_t max_vertex,
+                                                       VkDeviceSize stride /*= 3 * sizeof(float)*/) {
+    triangles_.host_vertex_buffer = std::move(vertex_buffer);
+    vk_obj_.geometry.triangles.vertexFormat = VK_FORMAT_R32G32B32_SFLOAT;
+    vk_obj_.geometry.triangles.vertexData.hostAddress = triangles_.host_vertex_buffer.get();
+    vk_obj_.geometry.triangles.maxVertex = max_vertex;
+    vk_obj_.geometry.triangles.vertexStride = stride;
+    return *this;
+}
+
+GeometryKHR &GeometryKHR::SetTrianglesDeviceIndexBuffer(vk_testing::Buffer &&index_buffer,
+                                                        VkIndexType index_type /*= VK_INDEX_TYPE_UINT32*/) {
+    triangles_.device_index_buffer = std::move(index_buffer);
     vk_obj_.geometry.triangles.indexType = index_type;
-    vk_obj_.geometry.triangles.indexData.deviceAddress = triangles_.index_buffer.address(vk_api_version_);
+    vk_obj_.geometry.triangles.indexData.deviceAddress = triangles_.device_index_buffer.address(vk_api_version_);
+    return *this;
+}
+
+GeometryKHR &GeometryKHR::SetTrianglesHostIndexBuffer(std::unique_ptr<uint32_t[]> index_buffer) {
+    triangles_.host_index_buffer = std::move(index_buffer);
+    vk_obj_.geometry.triangles.indexType = VK_INDEX_TYPE_UINT32;
+    vk_obj_.geometry.triangles.indexData.hostAddress = triangles_.host_index_buffer.get();
     return *this;
 }
 
@@ -86,13 +116,57 @@ GeometryKHR &GeometryKHR::SetTrianglesIndexType(VkIndexType index_type) {
     return *this;
 }
 
-GeometryKHR &GeometryKHR::SetAABBsBuffer(vk_testing::Buffer &&buffer, VkDeviceSize stride /*= sizeof(VkAabbPositionsKHR)*/) {
-    aabbs_.buffer = std::move(buffer);
-    vk_obj_.geometry.aabbs.data.deviceAddress = aabbs_.buffer.address(vk_api_version_);
+GeometryKHR &GeometryKHR::SetAABBsDeviceBuffer(vk_testing::Buffer &&buffer, VkDeviceSize stride /*= sizeof(VkAabbPositionsKHR)*/) {
+    aabbs_.device_buffer = std::move(buffer);
+    vk_obj_.geometry.aabbs.data.deviceAddress = aabbs_.device_buffer.address(vk_api_version_);
     vk_obj_.geometry.aabbs.stride = stride;
     return *this;
 }
 
+GeometryKHR &GeometryKHR::SetAABBsHostBuffer(std::unique_ptr<VkAabbPositionsKHR[]> buffer,
+                                             VkDeviceSize stride /*= sizeof(VkAabbPositionsKHR)*/) {
+    aabbs_.host_buffer = std::move(buffer);
+    vk_obj_.geometry.aabbs.data.hostAddress = aabbs_.host_buffer.get();
+    vk_obj_.geometry.aabbs.stride = stride;
+    return *this;
+}
+
+GeometryKHR &GeometryKHR::SetInstanceDeviceAccelStructRef(const vk_testing::Device &device,
+                                                          VkAccelerationStructureKHR bottom_level_as) {
+    auto vkGetAccelerationStructureDeviceAddressKHR = reinterpret_cast<PFN_vkGetAccelerationStructureDeviceAddressKHR>(
+        vk::GetDeviceProcAddr(device.handle(), "vkGetAccelerationStructureDeviceAddressKHR"));
+    assert(vkGetAccelerationStructureDeviceAddressKHR);
+    auto as_address_info = LvlInitStruct<VkAccelerationStructureDeviceAddressInfoKHR>();
+    as_address_info.accelerationStructure = bottom_level_as;
+    VkDeviceAddress as_address = vkGetAccelerationStructureDeviceAddressKHR(device.handle(), &as_address_info);
+    instance_.vk_instance = std::make_unique<VkAccelerationStructureInstanceKHR>();
+    instance_.vk_instance->accelerationStructureReference = static_cast<uint64_t>(as_address);
+    // leave other instance_ attributes to 0
+
+    // Create instance buffer. Do not copy instance_.vk_instance into it, for now no point in doing it for the test framework
+    assert(!instance_.buffer.initialized());  // for now, do not handle already initialized buffer
+    auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
+    alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
+    instance_.buffer.init(
+        device, sizeof(VkAccelerationStructureInstanceKHR), VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
+        VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_BUILD_INPUT_READ_ONLY_BIT_KHR,
+        &alloc_flags);
+
+    vk_obj_.geometry.instances.arrayOfPointers = VK_FALSE;
+    vk_obj_.geometry.instances.data.deviceAddress = instance_.buffer.address(vk_api_version_);
+    return *this;
+}
+
+GeometryKHR &GeometryKHR::SetInstanceHostAccelStructRef(VkAccelerationStructureKHR bottom_level_as) {
+    instance_.vk_instance = std::make_unique<VkAccelerationStructureInstanceKHR>();
+    instance_.vk_instance->accelerationStructureReference = (uint64_t)(bottom_level_as);
+    // leave other instance_ attributes to 0
+
+    vk_obj_.geometry.instances.arrayOfPointers = VK_FALSE;
+    vk_obj_.geometry.instances.data.hostAddress = instance_.vk_instance.get();
+    return *this;
+}
+
 GeometryKHR &GeometryKHR::Build() { return *this; }
 
 VkAccelerationStructureBuildRangeInfoKHR GeometryKHR::GetFullBuildRange() const {
@@ -106,7 +180,7 @@ VkAccelerationStructureBuildRangeInfoKHR GeometryKHR::GetFullBuildRange() const
 }
 
 AccelerationStructureKHR::AccelerationStructureKHR(uint32_t vk_api_version)
-    : vk_api_version_(vk_api_version), vk_info_(LvlInitStruct<decltype(vk_info_)>()), buffer_() {}
+    : vk_api_version_(vk_api_version), vk_info_(LvlInitStruct<decltype(vk_info_)>()), device_buffer_() {}
 
 AccelerationStructureKHR &AccelerationStructureKHR::SetSize(VkDeviceSize size) {
     vk_info_.size = size;
@@ -128,18 +202,25 @@ AccelerationStructureKHR &AccelerationStructureKHR::SetFlags(VkAccelerationStruc
     return *this;
 }
 
-AccelerationStructureKHR &AccelerationStructureKHR::SetBuffer(vk_testing::Buffer &&buffer) {
-    buffer_ = std::move(buffer);
+AccelerationStructureKHR &AccelerationStructureKHR::SetDeviceBuffer(vk_testing::Buffer &&buffer) {
+    device_buffer_ = std::move(buffer);
     return *this;
 }
 
-AccelerationStructureKHR &AccelerationStructureKHR::SetBufferMemoryAllocateFlags(VkMemoryAllocateFlags memory_allocate_flags) {
+AccelerationStructureKHR &AccelerationStructureKHR::SetDeviceBufferMemoryAllocateFlags(
+    VkMemoryAllocateFlags memory_allocate_flags) {
     buffer_memory_allocate_flags_ = memory_allocate_flags;
     return *this;
 }
 
-AccelerationStructureKHR &AccelerationStructureKHR::SetBufferMemoryPropertyFlags(VkMemoryAllocateFlags memory_property_flags_) {
-    buffer_memory_property_flags_ = memory_property_flags_;
+AccelerationStructureKHR &AccelerationStructureKHR::SetDeviceBufferMemoryPropertyFlags(
+    VkMemoryAllocateFlags memory_property_flags) {
+    buffer_memory_property_flags_ = memory_property_flags;
+    return *this;
+}
+
+AccelerationStructureKHR &AccelerationStructureKHR::SetDeviceBufferInitNoMem(bool buffer_init_no_mem) {
+    buffer_init_no_mem_ = buffer_init_no_mem;
     return *this;
 }
 
@@ -150,21 +231,28 @@ AccelerationStructureKHR &AccelerationStructureKHR::SetBufferUsageFlags(VkBuffer
 
 VkDeviceAddress AccelerationStructureKHR::GetBufferDeviceAddress() const {
     assert(initialized());
-    assert(buffer_.initialized());
-    assert(buffer_.create_info().size > 0);
-    return buffer_.address(vk_api_version_);
+    assert(device_buffer_.initialized());
+    assert(device_buffer_.create_info().size > 0);
+    return device_buffer_.address(vk_api_version_);
 }
 
 void AccelerationStructureKHR::Build(const vk_testing::Device &device) {
     assert(handle() == VK_NULL_HANDLE);
 
     // Create a buffer to store acceleration structure
-    if (!buffer_.initialized() && (buffer_usage_flags_ != 0)) {
+    if (!device_buffer_.initialized() && (buffer_usage_flags_ != 0)) {
         auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
         alloc_flags.flags = buffer_memory_allocate_flags_;
-        buffer_.init(device, vk_info_.size, buffer_memory_property_flags_, buffer_usage_flags_, &alloc_flags);
+        if (buffer_init_no_mem_) {
+            auto ci = LvlInitStruct<VkBufferCreateInfo>();
+            ci.size = vk_info_.size;
+            ci.usage = buffer_usage_flags_;
+            device_buffer_.init_no_mem(device, ci);
+        } else {
+            device_buffer_.init(device, vk_info_.size, buffer_memory_property_flags_, buffer_usage_flags_, &alloc_flags);
+        }
     }
-    vk_info_.buffer = buffer_.handle();
+    vk_info_.buffer = device_buffer_.handle();
 
     // Create acceleration structure
     auto vkCreateAccelerationStructureKHR = reinterpret_cast<PFN_vkCreateAccelerationStructureKHR>(
@@ -189,7 +277,7 @@ void AccelerationStructureKHR::Destroy() {
     assert(vkDestroyAccelerationStructureKHR);
     vkDestroyAccelerationStructureKHR(device(), handle(), nullptr);
     handle_ = VK_NULL_HANDLE;
-    buffer_.destroy();
+    device_buffer_.destroy();
 }
 
 BuildGeometryInfoKHR::BuildGeometryInfoKHR(uint32_t vk_api_version)
@@ -198,7 +286,7 @@ BuildGeometryInfoKHR::BuildGeometryInfoKHR(uint32_t vk_api_version)
       geometries_(),
       src_as_(std::make_shared<AccelerationStructureKHR>(vk_api_version)),
       dst_as_(std::make_shared<AccelerationStructureKHR>(vk_api_version)),
-      scratch_() {}
+      device_scratch_() {}
 
 BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetGeometries(std::vector<GeometryKHR> &&geometries) {
     geometries_ = std::move(geometries);
@@ -240,7 +328,12 @@ BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetDstAS(std::shared_ptr<Acceleratio
 }
 
 BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetScratchBuffer(vk_testing::Buffer &&scratch_buffer) {
-    scratch_ = std::move(scratch_buffer);
+    device_scratch_ = std::move(scratch_buffer);
+    return *this;
+}
+
+BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetBottomLevelAS(std::shared_ptr<BuildGeometryInfoKHR> bottom_level_as) {
+    bottom_level_as_ = std::move(bottom_level_as);
     return *this;
 }
 
@@ -260,19 +353,29 @@ BuildGeometryInfoKHR &BuildGeometryInfoKHR::SetNullBuildRangeInfos(bool use_null
     return *this;
 }
 
-void BuildGeometryInfoKHR::BuildCmdBuffer(const vk_testing::Device &device, VkCommandBuffer cmd_buffer,
+void BuildGeometryInfoKHR::BuildCmdBuffer(VkInstance instance, const vk_testing::Device &device, VkCommandBuffer cmd_buffer,
                                           bool use_ppGeometries /*= true*/) {
-    BuildCommon(device);
+    if (bottom_level_as_) {
+        bottom_level_as_->BuildCmdBuffer(instance, device, cmd_buffer, use_ppGeometries);
+    }
+    BuildCommon(instance, device, true);
     VkCmdBuildAccelerationStructuresKHR(device, cmd_buffer, true);
 }
 
-void BuildGeometryInfoKHR::BuildCmdBufferIndirect(const vk_testing::Device &device, VkCommandBuffer cmd_buffer) {
-    BuildCommon(device);
+void BuildGeometryInfoKHR::BuildCmdBufferIndirect(VkInstance instance, const vk_testing::Device &device,
+                                                  VkCommandBuffer cmd_buffer) {
+    if (bottom_level_as_) {
+        bottom_level_as_->BuildCmdBufferIndirect(instance, device, cmd_buffer);
+    }
+    BuildCommon(instance, device, true);
     VkCmdBuildAccelerationStructuresIndirectKHR(device, cmd_buffer);
 }
 
 void BuildGeometryInfoKHR::BuildHost(VkInstance instance, const vk_testing::Device &device) {
-    BuildCommon(device);
+    if (bottom_level_as_) {
+        bottom_level_as_->BuildHost(instance, device);
+    }
+    BuildCommon(instance, device, false);
     VkBuildAccelerationStructuresKHR(instance, device);
 }
 
@@ -324,12 +427,11 @@ void BuildGeometryInfoKHR::VkCmdBuildAccelerationStructuresIndirectKHR(const vk_
                                                                        VkCommandBuffer cmd_buffer) {
     // fill vk_info_ with geometry data, and get build ranges
     std::vector<const VkAccelerationStructureGeometryKHR *> pGeometries(geometries_.size());
-    std::vector<VkAccelerationStructureBuildRangeInfoKHR> range_infos(geometries_.size());
+
     pGeometries.reserve(geometries_.size());
     for (size_t i = 0; i < geometries_.size(); ++i) {
         const auto &geometry = geometries_[i];
         pGeometries[i] = &geometry.GetVkObj();
-        range_infos[i] = geometry.GetFullBuildRange();
     }
     vk_info_.geometryCount = static_cast<uint32_t>(geometries_.size());
     vk_info_.ppGeometries = pGeometries.data();
@@ -418,7 +520,8 @@ VkAccelerationStructureBuildSizesInfoKHR BuildGeometryInfoKHR::GetSizeInfo(VkDev
     return size_info;
 }
 
-void BuildGeometryInfoKHR::BuildCommon(const vk_testing::Device &device, bool use_ppGeometries /*= true*/) {
+void BuildGeometryInfoKHR::BuildCommon(VkInstance instance, const vk_testing::Device &device, bool is_on_device_build,
+                                       bool use_ppGeometries /*= true*/) {
     assert(vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR ||
            vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_UPDATE_KHR);
 
@@ -437,28 +540,53 @@ void BuildGeometryInfoKHR::BuildCommon(const vk_testing::Device &device, bool us
     }
     vk_info_.dstAccelerationStructure = dst_as_->handle();
 
-    // Allocate scratch buffer
-    if (!scratch_.initialized()) {
-        auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
-        alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
-        const VkAccelerationStructureBuildSizesInfoKHR size_info = GetSizeInfo(device.handle(), use_ppGeometries);
-        const VkDeviceSize scratch_size = vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR
-                                              ? size_info.buildScratchSize
-                                              : size_info.updateScratchSize;
-        if (scratch_size != 0) {
-            scratch_.init(device, scratch_size, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
-                          VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, &alloc_flags);
+    const VkAccelerationStructureBuildSizesInfoKHR size_info = GetSizeInfo(device.handle(), use_ppGeometries);
+    const VkDeviceSize scratch_size =
+        vk_info_.mode == VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR ? size_info.buildScratchSize : size_info.updateScratchSize;
+    if (is_on_device_build) {
+        // Allocate device local scratch buffer
+
+        // Get minAccelerationStructureScratchOffsetAlignment
+        auto vkGetPhysicalDeviceProperties2 = reinterpret_cast<PFN_vkGetPhysicalDeviceProperties2>(
+            vk::GetInstanceProcAddr(instance, "vkGetPhysicalDeviceProperties2"));
+        assert(vkGetPhysicalDeviceProperties2);
+        auto as_props = LvlInitStruct<VkPhysicalDeviceAccelerationStructurePropertiesKHR>();
+        auto phys_dev_props = LvlInitStruct<VkPhysicalDeviceProperties2>(&as_props);
+        vkGetPhysicalDeviceProperties2(device.phy(), &phys_dev_props);
+
+        if (!device_scratch_.initialized()) {
+            auto alloc_flags = LvlInitStruct<VkMemoryAllocateFlagsInfo>();
+            alloc_flags.flags = VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR;
+
+            if (scratch_size > 0) {
+                device_scratch_.init(device, scratch_size + as_props.minAccelerationStructureScratchOffsetAlignment,
+                                     VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT,
+                                     VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, &alloc_flags);
+            }
+        }
+        if (device_scratch_.create_info().size != 0 &&
+            device_scratch_.create_info().usage & VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT) {
+            const VkDeviceAddress scratch_address = device_scratch_.address(vk_api_version_);
+            const auto aligned_scratch_address =
+                Align<VkDeviceAddress>(scratch_address, as_props.minAccelerationStructureScratchOffsetAlignment);
+            assert(aligned_scratch_address >= scratch_address);
+            assert(aligned_scratch_address < (scratch_address + as_props.minAccelerationStructureScratchOffsetAlignment));
+            vk_info_.scratchData.deviceAddress = aligned_scratch_address;
+        } else {
+            vk_info_.scratchData.deviceAddress = 0;
         }
-    }
-    if (scratch_.create_info().size != 0 && scratch_.create_info().usage & VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT) {
-        vk_info_.scratchData.deviceAddress = scratch_.address(vk_api_version_);
     } else {
-        vk_info_.scratchData.deviceAddress = 0;
+        // Allocate on host scratch buffer
+        host_scratch_ = nullptr;
+        if (scratch_size > 0) {
+            host_scratch_ = std::make_unique<uint8_t[]>(scratch_size);
+        }
+        vk_info_.scratchData.hostAddress = host_scratch_.get();
     }
 }
 
 namespace blueprint {
-GeometryKHR GeometrySimpleTriangleInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
+GeometryKHR GeometrySimpleOnDeviceTriangleInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
     GeometryKHR triangle_geometry(vk_api_version);
 
     triangle_geometry.SetType(GeometryKHR::Type::Triangle);
@@ -478,24 +606,47 @@ GeometryKHR GeometrySimpleTriangleInfo(uint32_t vk_api_version, const vk_testing
     // Fill vertex and index buffers with one triangle
     triangle_geometry.SetPrimitiveCount(1);
     constexpr std::array vertices = {1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, -1.0f, 0.0f, 0.0f};
-    constexpr std::array<uint32_t, 3> indicies = {{0, 1, 2}};
+    constexpr std::array<uint32_t, 3> indices = {{0, 1, 2}};
 
     auto mapped_vbo_buffer_data = static_cast<float *>(vertex_buffer.memory().map());
     std::copy(vertices.begin(), vertices.end(), mapped_vbo_buffer_data);
     vertex_buffer.memory().unmap();
 
     auto mapped_ibo_buffer_data = static_cast<uint32_t *>(index_buffer.memory().map());
-    std::copy(indicies.begin(), indicies.end(), mapped_ibo_buffer_data);
+    std::copy(indices.begin(), indices.end(), mapped_ibo_buffer_data);
     index_buffer.memory().unmap();
 
     // Assign vertex and index buffers to out geometry
-    triangle_geometry.SetTrianglesVertexBuffer(std::move(vertex_buffer), uint32_t(vertices.size() / 3 - 1));
-    triangle_geometry.SetTrianglesIndexBuffer(std::move(index_buffer));
+    triangle_geometry.SetTrianglesDeviceVertexBuffer(std::move(vertex_buffer), uint32_t(vertices.size() / 3 - 1));
+    triangle_geometry.SetTrianglesDeviceIndexBuffer(std::move(index_buffer));
 
     return triangle_geometry;
 }
 
-GeometryKHR GeometrySimpleAABBInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
+GeometryKHR GeometrySimpleOnHostTriangleInfo(uint32_t vk_api_version) {
+    GeometryKHR triangle_geometry(vk_api_version);
+
+    triangle_geometry.SetType(GeometryKHR::Type::Triangle);
+
+    // Fill vertex and index buffers with one triangle
+    triangle_geometry.SetPrimitiveCount(1);
+    constexpr std::array vertices = {1.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f, -1.0f, 0.0f, 0.0f};
+    constexpr std::array<uint32_t, 3> indices = {{0, 1, 2}};
+
+    auto vertex_buffer = std::make_unique<float[]>(vertices.size());
+    std::copy(vertices.data(), vertices.data() + vertices.size(), vertex_buffer.get());
+
+    auto index_buffer = std::make_unique<uint32_t[]>(indices.size());
+    std::copy(indices.data(), indices.data() + indices.size(), index_buffer.get());
+
+    // Assign vertex and index buffers to out geometry
+    triangle_geometry.SetTrianglesHostVertexBuffer(std::move(vertex_buffer), uint32_t(vertices.size() / 3 - 1));
+    triangle_geometry.SetTrianglesHostIndexBuffer(std::move(index_buffer));
+
+    return triangle_geometry;
+}
+
+GeometryKHR GeometrySimpleOnDeviceAABBInfo(uint32_t vk_api_version, const vk_testing::Device &device) {
     GeometryKHR aabb_geometry(vk_api_version);
 
     aabb_geometry.SetType(GeometryKHR::Type::AABB);
@@ -520,11 +671,49 @@ GeometryKHR GeometrySimpleAABBInfo(uint32_t vk_api_version, const vk_testing::De
     std::copy(aabbs.begin(), aabbs.end(), mapped_aabb_buffer_data);
     aabb_buffer.memory().unmap();
 
-    aabb_geometry.SetAABBsBuffer(std::move(aabb_buffer));
+    aabb_geometry.SetAABBsDeviceBuffer(std::move(aabb_buffer));
 
     return aabb_geometry;
 }
 
+GeometryKHR GeometrySimpleOnHostAABBInfo(uint32_t vk_api_version) {
+    GeometryKHR aabb_geometry(vk_api_version);
+
+    aabb_geometry.SetType(GeometryKHR::Type::AABB);
+
+    // Fill buffer with one aabb
+    const std::array<VkAabbPositionsKHR, 1> aabbs = {{{-1.0f, -1.0f, -1.0f, +1.0f, +1.0f, +1.0f}}};
+
+    auto aabb_buffer = std::make_unique<VkAabbPositionsKHR[]>(aabbs.size());
+    std::copy(aabbs.data(), aabbs.data() + aabbs.size(), aabb_buffer.get());
+
+    // Assign aabb buffer to out geometry
+    aabb_geometry.SetAABBsHostBuffer(std::move(aabb_buffer));
+
+    return aabb_geometry;
+}
+
+GeometryKHR GeometrySimpleDeviceInstance(uint32_t vk_api_version, const vk_testing::Device &device,
+                                         VkAccelerationStructureKHR device_instance) {
+    GeometryKHR instance_geometry(vk_api_version);
+
+    instance_geometry.SetType(GeometryKHR::Type::Instance);
+    instance_geometry.SetPrimitiveCount(1);
+    instance_geometry.SetInstanceDeviceAccelStructRef(device, device_instance);
+
+    return instance_geometry;
+}
+
+GeometryKHR GeometrySimpleHostInstance(uint32_t vk_api_version, VkAccelerationStructureKHR host_instance) {
+    GeometryKHR instance_geometry(vk_api_version);
+
+    instance_geometry.SetType(GeometryKHR::Type::Instance);
+    instance_geometry.SetPrimitiveCount(1);
+    instance_geometry.SetInstanceHostAccelStructRef(host_instance);
+
+    return instance_geometry;
+}
+
 std::shared_ptr<AccelerationStructureKHR> AccelStructNull(uint32_t vk_api_version) {
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetNull(true);
@@ -535,10 +724,34 @@ std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceBottomLevel(u
     auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
     as->SetSize(size);
     as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
-    as->SetBufferMemoryAllocateFlags(VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR);
-    as->SetBufferMemoryPropertyFlags(VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+    as->SetDeviceBufferMemoryAllocateFlags(VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR);
+    as->SetDeviceBufferMemoryPropertyFlags(VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
     as->SetBufferUsageFlags(VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR | VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT |
                             VK_BUFFER_USAGE_STORAGE_BUFFER_BIT);
+    as->SetDeviceBufferInitNoMem(false);
+    return as;
+}
+
+std::shared_ptr<rt::as::AccelerationStructureKHR> AccelStructSimpleOnHostBottomLevel(uint32_t vk_api_version, VkDeviceSize size) {
+    auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
+    as->SetSize(size);
+    as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
+    as->SetDeviceBufferMemoryAllocateFlags(0);
+    as->SetDeviceBufferMemoryPropertyFlags(VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT);
+    as->SetBufferUsageFlags(VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR | VK_BUFFER_USAGE_STORAGE_BUFFER_BIT);
+    as->SetDeviceBufferInitNoMem(false);
+    return as;
+}
+
+std::shared_ptr<AccelerationStructureKHR> AccelStructSimpleOnDeviceTopLevel(uint32_t vk_api_version, VkDeviceSize size) {
+    auto as = std::make_shared<AccelerationStructureKHR>(vk_api_version);
+    as->SetSize(size);
+    as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
+    as->SetDeviceBufferMemoryAllocateFlags(VK_MEMORY_ALLOCATE_DEVICE_ADDRESS_BIT_KHR);
+    as->SetDeviceBufferMemoryPropertyFlags(VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT);
+    as->SetBufferUsageFlags(VK_BUFFER_USAGE_ACCELERATION_STRUCTURE_STORAGE_BIT_KHR | VK_BUFFER_USAGE_SHADER_DEVICE_ADDRESS_BIT |
+                            VK_BUFFER_USAGE_STORAGE_BUFFER_BIT);
+    as->SetDeviceBufferInitNoMem(false);
     return as;
 }
 
@@ -546,18 +759,20 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(uint32_t vk_api_
                                                                 GeometryKHR::Type geometry_type /*= GeometryKHR::Type::Triangle*/) {
     BuildGeometryInfoKHR out_build_info(vk_api_version);
 
-    out_build_info.SetMode(VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR);
     out_build_info.SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
+    out_build_info.SetMode(VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR);
 
     // Set geometry
     std::vector<GeometryKHR> geometries;
     switch (geometry_type) {
         case GeometryKHR::Type::Triangle:
-            geometries.emplace_back(GeometrySimpleTriangleInfo(vk_api_version, device));
+            geometries.emplace_back(GeometrySimpleOnDeviceTriangleInfo(vk_api_version, device));
             break;
         case GeometryKHR::Type::AABB:
-            geometries.emplace_back(GeometrySimpleAABBInfo(vk_api_version, device));
+            geometries.emplace_back(GeometrySimpleOnDeviceAABBInfo(vk_api_version, device));
             break;
+        case GeometryKHR::Type::Instance:
+            [[fallthrough]];
         case GeometryKHR::Type::_INTERNAL_UNSPECIFIED:
             assert(false);
             break;
@@ -576,6 +791,104 @@ BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceBottomLevel(uint32_t vk_api_
     return out_build_info;
 }
 
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostBottomLevel(uint32_t vk_api_version, const vk_testing::Device &device,
+                                                              GeometryKHR::Type geometry_type /*= GeometryKHR::Type::Triangle*/) {
+    BuildGeometryInfoKHR out_build_info(vk_api_version);
+
+    out_build_info.SetType(VK_ACCELERATION_STRUCTURE_TYPE_BOTTOM_LEVEL_KHR);
+    out_build_info.SetMode(VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR);
+
+    // Set geometry
+    std::vector<GeometryKHR> geometries;
+    switch (geometry_type) {
+        case GeometryKHR::Type::Triangle:
+            geometries.emplace_back(GeometrySimpleOnHostTriangleInfo(vk_api_version));
+            break;
+        case GeometryKHR::Type::AABB:
+            geometries.emplace_back(GeometrySimpleOnHostAABBInfo(vk_api_version));
+            break;
+        case GeometryKHR::Type::Instance:
+            [[fallthrough]];
+        case GeometryKHR::Type::_INTERNAL_UNSPECIFIED:
+            assert(false);
+            break;
+    }
+    out_build_info.SetGeometries(std::move(geometries));
+
+    // Set source and destination acceleration structures info. Does not create handles, it is done in Build()
+    out_build_info.SetSrcAS(AccelStructNull(vk_api_version));
+    auto dstAsSize = out_build_info.GetSizeInfo(device.handle()).accelerationStructureSize;
+    out_build_info.SetDstAS(AccelStructSimpleOnHostBottomLevel(vk_api_version, dstAsSize));
+
+    out_build_info.SetInfoCount(1);
+    out_build_info.SetNullInfos(false);
+    out_build_info.SetNullBuildRangeInfos(false);
+
+    return out_build_info;
+}
+
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnDeviceTopLevel(
+    uint32_t vk_api_version, const vk_testing::Device &device,
+    std::shared_ptr<BuildGeometryInfoKHR> on_device_bottom_level_geometry) {
+    BuildGeometryInfoKHR out_build_info(vk_api_version);
+
+    out_build_info.SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
+    out_build_info.SetMode(VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR);
+
+    // Set bottom level acceleration structure
+    assert(on_device_bottom_level_geometry->GetDstAS()->IsBuilt());
+    out_build_info.SetBottomLevelAS(std::move(on_device_bottom_level_geometry));
+
+    // Set geometry to one instance pointing to bottom level acceleration structure
+    std::vector<GeometryKHR> geometries;
+    geometries.emplace_back(
+        GeometrySimpleDeviceInstance(vk_api_version, device, out_build_info.GetBottomLevelAS()->GetDstAS()->handle()));
+    out_build_info.SetGeometries(std::move(geometries));
+
+    // Set source and destination acceleration structures info. Does not create handles, it is done in Build()
+    out_build_info.SetSrcAS(AccelStructNull(vk_api_version));
+    auto dstAsSize = out_build_info.GetSizeInfo(device.handle()).accelerationStructureSize;
+    auto dst_as = AccelStructSimpleOnDeviceBottomLevel(vk_api_version, dstAsSize);
+    dst_as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
+    out_build_info.SetDstAS(std::move(dst_as));
+
+    out_build_info.SetInfoCount(1);
+    out_build_info.SetNullInfos(false);
+    out_build_info.SetNullBuildRangeInfos(false);
+
+    return out_build_info;
+}
+
+BuildGeometryInfoKHR BuildGeometryInfoSimpleOnHostTopLevel(uint32_t vk_api_version, const vk_testing::Device &device,
+                                                           std::shared_ptr<BuildGeometryInfoKHR> on_host_bottom_level_geometry) {
+    BuildGeometryInfoKHR out_build_info(vk_api_version);
+
+    out_build_info.SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
+    out_build_info.SetMode(VK_BUILD_ACCELERATION_STRUCTURE_MODE_BUILD_KHR);
+
+    // Set bottom level acceleration structure
+    assert(on_host_bottom_level_geometry->GetDstAS()->IsBuilt());
+    out_build_info.SetBottomLevelAS(std::move(on_host_bottom_level_geometry));
+
+    // Set geometry to one instance pointing to bottom level acceleration structure
+    std::vector<GeometryKHR> geometries;
+    geometries.emplace_back(GeometrySimpleHostInstance(vk_api_version, out_build_info.GetBottomLevelAS()->GetDstAS()->handle()));
+    out_build_info.SetGeometries(std::move(geometries));
+
+    // Set source and destination acceleration structures info. Does not create handles, it is done in Build()
+    out_build_info.SetSrcAS(AccelStructNull(vk_api_version));
+    auto dstAsSize = out_build_info.GetSizeInfo(device.handle()).accelerationStructureSize;
+    auto dst_as = AccelStructSimpleOnHostBottomLevel(vk_api_version, dstAsSize);
+    dst_as->SetType(VK_ACCELERATION_STRUCTURE_TYPE_TOP_LEVEL_KHR);
+    out_build_info.SetDstAS(std::move(dst_as));
+
+    out_build_info.SetInfoCount(1);
+    out_build_info.SetNullInfos(false);
+    out_build_info.SetNullBuildRangeInfos(false);
+
+    return out_build_info;
+}
+
 }  // namespace blueprint
 
 }  // namespace as
