@@ -18,7 +18,7 @@
 
 #pragma once
 
-#include "lvt_function_pointers.h"
+#include "generated/lvt_function_pointers.h"
 #include "error_monitor.h"
 
 #ifdef ANDROID
@@ -234,6 +234,8 @@ class VkRenderFramework : public VkTestFramework {
     // Does not return anything as the caller should use AreRequiredExtensionsEnabled or AddOptionalExtensions then
     // `ext_name` can refer to a device or instance extension.
     void AddRequiredExtensions(const char *ext_name);
+    // Ensures at least 1 WSI instance extension is enabled
+    void AddWsiExtensions(const char *ext_name);
     // Same as AddRequiredExtensions but won't fail a check to AreRequiredExtensionsEnabled
     void AddOptionalExtensions(const char *ext_name);
     // After instance and physical device creation (e.g., after InitFramework), returns true if all required extensions are
@@ -326,6 +328,8 @@ class VkRenderFramework : public VkTestFramework {
     std::vector<const char *> m_required_extensions;
     // Optional extensions to try and enable at device creation time
     std::vector<const char *> m_optional_extensions;
+    // wsi extensions to try and enable
+    std::vector<const char *> m_wsi_extensions;
     // Device extensions to enable
     std::vector<const char *> m_device_extension_names;
 
@@ -363,7 +367,7 @@ typedef vk_testing::Fence VkFenceObj;
 typedef vk_testing::Semaphore VkSemaphoreObj;
 typedef vk_testing::Buffer VkBufferObj;
 typedef vk_testing::AccelerationStructure VkAccelerationStructureObj;
-typedef vk_testing::AccelerationStructureKHR VkAccelerationStructurekhrObj;
+using VkDeviceMemoryObj = vk_testing::DeviceMemory;
 class VkCommandPoolObj : public vk_testing::CommandPool {
   public:
     VkCommandPoolObj() : vk_testing::CommandPool(){};
@@ -518,7 +522,7 @@ class VkImageObj : public vk_testing::Image {
         return ci;
     }
 
-    VkImageView targetView(VkImageViewCreateInfo ci) {
+    const VkImageView &targetView(VkImageViewCreateInfo ci) {
         if (!m_targetView.initialized()) {
             ci.image = handle();
             m_targetView.init(*m_device, ci);
@@ -526,9 +530,9 @@ class VkImageObj : public vk_testing::Image {
         return m_targetView.handle();
     }
 
-    VkImageView targetView(VkFormat format, VkImageAspectFlags aspect = VK_IMAGE_ASPECT_COLOR_BIT, uint32_t baseMipLevel = 0,
-                           uint32_t levelCount = VK_REMAINING_MIP_LEVELS, uint32_t baseArrayLayer = 0,
-                           uint32_t layerCount = VK_REMAINING_ARRAY_LAYERS, VkImageViewType type = VK_IMAGE_VIEW_TYPE_2D) {
+    const VkImageView &targetView(VkFormat format, VkImageAspectFlags aspect = VK_IMAGE_ASPECT_COLOR_BIT, uint32_t baseMipLevel = 0,
+                                  uint32_t levelCount = VK_REMAINING_MIP_LEVELS, uint32_t baseArrayLayer = 0,
+                                  uint32_t layerCount = VK_REMAINING_ARRAY_LAYERS, VkImageViewType type = VK_IMAGE_VIEW_TYPE_2D) {
         if (!m_targetView.initialized()) {
             VkImageViewCreateInfo createView = LvlInitStruct<VkImageViewCreateInfo>();
             createView.image = handle();
