@@ -12,10 +12,10 @@
  *     http://www.apache.org/licenses/LICENSE-2.0
  */
 
-#include "cast_utils.h"
-#include "enum_flag_bits.h"
+#include "utils/cast_utils.h"
+#include "generated/enum_flag_bits.h"
 #include "../framework/layer_validation_tests.h"
-#include "vk_layer_utils.h"
+#include "utils/vk_layer_utils.h"
 
 TEST_F(VkLayerTest, CreateBufferIncompatibleExternalHandleTypes) {
     TEST_DESCRIPTION("Creating buffer with incompatible external memory handle types");
@@ -1096,6 +1096,7 @@ TEST_F(VkLayerTest, ValidateImportMemoryHandleType) {
     buffer_info.pNext = &external_buffer_info;
     VkBufferObj buffer_export;
     buffer_export.init_no_mem(*m_device, buffer_info);
+    const VkMemoryRequirements buffer_export_reqs = buffer_export.memory_requirements();
 
     auto importable_buffer_types =
         FindSupportedExternalMemoryHandleTypes(gpu(), buffer_info, VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT);
@@ -1107,9 +1108,15 @@ TEST_F(VkLayerTest, ValidateImportMemoryHandleType) {
 
     VkBufferObj buffer_import;
     buffer_import.init_no_mem(*m_device, buffer_info);
+    const VkMemoryRequirements buffer_import_reqs = buffer_import.memory_requirements();
+    assert(buffer_import_reqs.memoryTypeBits != 0);  // according to spec at least one bit is set
+    if ((buffer_import_reqs.memoryTypeBits & buffer_export_reqs.memoryTypeBits) == 0) {
+        // required by VU 01743
+        GTEST_SKIP() << "Cannot find memory type that supports both export and import";
+    }
 
     // Allocation info
-    auto alloc_info = vk_testing::DeviceMemory::get_resource_alloc_info(*m_device, buffer_export.memory_requirements(), mem_flags);
+    auto alloc_info = vk_testing::DeviceMemory::get_resource_alloc_info(*m_device, buffer_export_reqs, mem_flags);
 
     // Add export allocation info to pNext chain
     auto export_info = LvlInitStruct<VkExportMemoryAllocateInfoKHR>();
@@ -1215,10 +1222,6 @@ TEST_F(VkLayerTest, ValidateImportMemoryHandleType) {
 #endif
 
     // Import memory
-    VkMemoryRequirements buffer_import_reqs = buffer_import.memory_requirements();
-    if (buffer_import_reqs.memoryTypeBits == 0) {
-        GTEST_SKIP() << "no suitable memory found, skipping test";
-    }
     alloc_info = vk_testing::DeviceMemory::get_resource_alloc_info(*m_device, buffer_import_reqs, mem_flags);
     alloc_info.pNext = &import_info_buffer;
     vk_testing::DeviceMemory memory_buffer_import;
@@ -1503,3 +1506,270 @@ TEST_F(VkLayerTest, InvalidD3D12FenceSubmitInfo) {
     }
 }
 #endif  // VK_USE_PLATFORM_WIN32_KHR
+
+TEST_F(VkLayerTest, GetMemoryFdHandle) {
+    TEST_DESCRIPTION("Validate VkMemoryGetFdInfoKHR passed to vkGetMemoryFdKHR");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_KHR_EXTERNAL_MEMORY_FD_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (DeviceValidationVersion() < VK_API_VERSION_1_1) {
+        GTEST_SKIP() << "At least Vulkan version 1.1 is required";
+    }
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
+    ASSERT_NO_FATAL_FAILURE(InitState());
+    auto vkGetMemoryFdKHR = GetInstanceProcAddr<PFN_vkGetMemoryFdKHR>("vkGetMemoryFdKHR");
+    int fd = -1;
+
+    // Allocate memory without VkExportMemoryAllocateInfo in the pNext chain
+    {
+        auto alloc_info = LvlInitStruct<VkMemoryAllocateInfo>();
+        alloc_info.allocationSize = 32;
+        alloc_info.memoryTypeIndex = 0;
+        vk_testing::DeviceMemory memory;
+        memory.init(*m_device, alloc_info);
+
+        auto get_handle_info = LvlInitStruct<VkMemoryGetFdInfoKHR>();
+        get_handle_info.memory = memory;
+        get_handle_info.handleType = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
+
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryGetFdInfoKHR-handleType-00671");
+        vkGetMemoryFdKHR(*m_device, &get_handle_info, &fd);
+        m_errorMonitor->VerifyFound();
+    }
+    // VkExportMemoryAllocateInfo::handleTypes does not include requested handle type
+    {
+        auto export_info = LvlInitStruct<VkExportMemoryAllocateInfo>();
+        export_info.handleTypes = 0;
+
+        auto alloc_info = LvlInitStruct<VkMemoryAllocateInfo>(&export_info);
+        alloc_info.allocationSize = 1024;
+        alloc_info.memoryTypeIndex = 0;
+        vk_testing::DeviceMemory memory;
+        memory.init(*m_device, alloc_info);
+
+        auto get_handle_info = LvlInitStruct<VkMemoryGetFdInfoKHR>();
+        get_handle_info.memory = memory;
+        get_handle_info.handleType = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
+
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryGetFdInfoKHR-handleType-00671");
+        vkGetMemoryFdKHR(*m_device, &get_handle_info, &fd);
+        m_errorMonitor->VerifyFound();
+    }
+    // Request handle of the wrong type
+    {
+        auto export_info = LvlInitStruct<VkExportMemoryAllocateInfo>();
+        export_info.handleTypes = VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_BIT;
+
+        auto alloc_info = LvlInitStruct<VkMemoryAllocateInfo>(&export_info);
+        alloc_info.allocationSize = 1024;
+        alloc_info.memoryTypeIndex = 0;
+
+        vk_testing::DeviceMemory memory;
+        memory.init(*m_device, alloc_info);
+        auto get_handle_info = LvlInitStruct<VkMemoryGetFdInfoKHR>();
+        get_handle_info.memory = memory;
+        get_handle_info.handleType = VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_BIT;
+
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryGetFdInfoKHR-handleType-00672");
+        vkGetMemoryFdKHR(*m_device, &get_handle_info, &fd);
+        m_errorMonitor->VerifyFound();
+    }
+}
+
+TEST_F(VkLayerTest, ImportMemoryFromFdHandle) {
+    TEST_DESCRIPTION("POSIX fd handle memory import. Import parameters do not match payload's parameters");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_KHR_EXTERNAL_MEMORY_FD_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (DeviceValidationVersion() < VK_API_VERSION_1_1) {
+        GTEST_SKIP() << "At least Vulkan version 1.1 is required";
+    }
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
+    ASSERT_NO_FATAL_FAILURE(InitState());
+    auto vkGetMemoryFdKHR = GetInstanceProcAddr<PFN_vkGetMemoryFdKHR>("vkGetMemoryFdKHR");
+    constexpr auto handle_type = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_FD_BIT;
+
+    VkExternalMemoryFeatureFlags external_features = 0;
+    {
+        constexpr auto required_features =
+            VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT_KHR | VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT_KHR;
+        auto external_info = LvlInitStruct<VkPhysicalDeviceExternalBufferInfo>();
+        external_info.usage = VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT;
+        external_info.handleType = handle_type;
+        auto external_properties = LvlInitStruct<VkExternalBufferProperties>();
+        vk::GetPhysicalDeviceExternalBufferProperties(gpu(), &external_info, &external_properties);
+        external_features = external_properties.externalMemoryProperties.externalMemoryFeatures;
+        if ((external_features & required_features) != required_features) {
+            GTEST_SKIP() << "External buffer does not support both export and import, skipping test";
+        }
+    }
+
+    VkBufferObj buffer;
+    {
+        auto external_info = LvlInitStruct<VkExternalMemoryBufferCreateInfo>();
+        external_info.handleTypes = handle_type;
+        auto create_info = VkBufferObj::create_info(1024, VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT);
+        create_info.pNext = &external_info;
+        buffer.init_no_mem(*m_device, create_info);
+    }
+
+    VkDeviceMemoryObj memory;
+    VkDeviceSize payload_size = 0;
+    uint32_t payload_memory_type = 0;
+    {
+        const bool dedicated_allocation = (external_features & VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT) != 0;
+        auto dedicated_info = LvlInitStruct<VkMemoryDedicatedAllocateInfo>();
+        dedicated_info.buffer = buffer;
+        auto export_info = LvlInitStruct<VkExportMemoryAllocateInfo>(dedicated_allocation ? &dedicated_info : nullptr);
+        export_info.handleTypes = handle_type;
+        auto alloc_info = VkDeviceMemoryObj::get_resource_alloc_info(*m_device, buffer.memory_requirements(), 0, &export_info);
+        memory.init(*m_device, alloc_info);
+        buffer.bind_memory(memory, 0);
+        payload_size = alloc_info.allocationSize;
+        payload_memory_type = alloc_info.memoryTypeIndex;
+    }
+
+    int fd = -1;
+    {
+        auto get_handle_info = LvlInitStruct<VkMemoryGetFdInfoKHR>();
+        get_handle_info.memory = memory;
+        get_handle_info.handleType = handle_type;
+        ASSERT_VK_SUCCESS(vkGetMemoryFdKHR(*m_device, &get_handle_info, &fd));
+    }
+    auto import_info = LvlInitStruct<VkImportMemoryFdInfoKHR>();
+    import_info.handleType = handle_type;
+    import_info.fd = fd;
+    auto alloc_info_with_import = LvlInitStruct<VkMemoryAllocateInfo>(&import_info);
+    VkDeviceMemory imported_memory = VK_NULL_HANDLE;
+
+    // allocationSize != payload's allocationSize
+    {
+        alloc_info_with_import.allocationSize = payload_size * 2;
+        alloc_info_with_import.memoryTypeIndex = payload_memory_type;
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryAllocateInfo-allocationSize-01742");
+        vk::AllocateMemory(*m_device, &alloc_info_with_import, nullptr, &imported_memory);
+        m_errorMonitor->VerifyFound();
+    }
+    // memoryTypeIndex != payload's memoryTypeIndex
+    {
+        alloc_info_with_import.allocationSize = payload_size;
+        alloc_info_with_import.memoryTypeIndex = payload_memory_type + 1;
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryAllocateInfo-allocationSize-01742");
+        vk::AllocateMemory(*m_device, &alloc_info_with_import, nullptr, &imported_memory);
+        m_errorMonitor->VerifyFound();
+    }
+    // Finish this test with a successful import operation in order to release the ownership of the file descriptor.
+    // The alternative is to use 'close' system call.
+    {
+        alloc_info_with_import.allocationSize = payload_size;
+        alloc_info_with_import.memoryTypeIndex = payload_memory_type;
+        vk_testing::DeviceMemory successfully_imported_memory;
+        successfully_imported_memory.init(*m_device, alloc_info_with_import);
+    }
+}
+
+#ifdef VK_USE_PLATFORM_WIN32_KHR
+TEST_F(VkLayerTest, ImportMemoryFromWin32Handle) {
+    TEST_DESCRIPTION("Win32 handle memory import. Import parameters do not match payload's parameters");
+    SetTargetApiVersion(VK_API_VERSION_1_1);
+    AddRequiredExtensions(VK_KHR_EXTERNAL_MEMORY_WIN32_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (DeviceValidationVersion() < VK_API_VERSION_1_1) {
+        GTEST_SKIP() << "At least Vulkan version 1.1 is required";
+    }
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
+    ASSERT_NO_FATAL_FAILURE(InitState());
+    auto vkGetMemoryWin32HandleKHR = GetInstanceProcAddr<PFN_vkGetMemoryWin32HandleKHR>("vkGetMemoryWin32HandleKHR");
+    constexpr auto handle_type = VK_EXTERNAL_MEMORY_HANDLE_TYPE_OPAQUE_WIN32_BIT;
+
+    VkExternalMemoryFeatureFlags external_features = 0;
+    {
+        constexpr auto required_features =
+            VK_EXTERNAL_MEMORY_FEATURE_EXPORTABLE_BIT_KHR | VK_EXTERNAL_MEMORY_FEATURE_IMPORTABLE_BIT_KHR;
+        auto external_info = LvlInitStruct<VkPhysicalDeviceExternalImageFormatInfo>();
+        external_info.handleType = handle_type;
+        auto image_info = LvlInitStruct<VkPhysicalDeviceImageFormatInfo2>(&external_info);
+        image_info.format = VK_FORMAT_R8G8B8A8_UNORM;
+        image_info.type = VK_IMAGE_TYPE_2D;
+        image_info.tiling = VK_IMAGE_TILING_OPTIMAL;
+        image_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
+        auto external_properties = LvlInitStruct<VkExternalImageFormatProperties>();
+        auto image_properties = LvlInitStruct<VkImageFormatProperties2>(&external_properties);
+        const VkResult result = vk::GetPhysicalDeviceImageFormatProperties2(gpu(), &image_info, &image_properties);
+        external_features = external_properties.externalMemoryProperties.externalMemoryFeatures;
+        if (result != VK_SUCCESS || (external_features & required_features) != required_features) {
+            GTEST_SKIP() << "External image does not support both export and import, skipping test";
+        }
+    }
+
+    VkImageObj image(m_device);
+    {
+        auto external_info = LvlInitStruct<VkExternalMemoryImageCreateInfo>();
+        external_info.handleTypes = handle_type;
+        auto create_info = VkImageObj::create_info();
+        create_info.pNext = &external_info;
+        create_info.imageType = VK_IMAGE_TYPE_2D;
+        create_info.format = VK_FORMAT_R8G8B8A8_UNORM;
+        create_info.tiling = VK_IMAGE_TILING_OPTIMAL;
+        create_info.usage = VK_IMAGE_USAGE_SAMPLED_BIT;
+        image.init_no_mem(*m_device, create_info);
+    }
+
+    VkDeviceMemoryObj memory;
+    VkDeviceSize payload_size = 0;
+    uint32_t payload_memory_type = 0;
+    {
+        const bool dedicated_allocation = (external_features & VK_EXTERNAL_MEMORY_FEATURE_DEDICATED_ONLY_BIT) != 0;
+        auto dedicated_info = LvlInitStruct<VkMemoryDedicatedAllocateInfo>();
+        dedicated_info.image = image;
+        auto export_info = LvlInitStruct<VkExportMemoryAllocateInfo>(dedicated_allocation ? &dedicated_info : nullptr);
+        export_info.handleTypes = handle_type;
+        auto alloc_info = VkDeviceMemoryObj::get_resource_alloc_info(*m_device, image.memory_requirements(), 0, &export_info);
+        memory.init(*m_device, alloc_info);
+        image.bind_memory(memory, 0);
+        payload_size = alloc_info.allocationSize;
+        payload_memory_type = alloc_info.memoryTypeIndex;
+    }
+
+    HANDLE handle = NULL;
+    {
+        auto get_handle_info = LvlInitStruct<VkMemoryGetWin32HandleInfoKHR>();
+        get_handle_info.memory = memory;
+        get_handle_info.handleType = handle_type;
+        ASSERT_VK_SUCCESS(vkGetMemoryWin32HandleKHR(*m_device, &get_handle_info, &handle));
+    }
+    auto import_info = LvlInitStruct<VkImportMemoryWin32HandleInfoKHR>();
+    import_info.handleType = handle_type;
+    import_info.handle = handle;
+    auto alloc_info_with_import = LvlInitStruct<VkMemoryAllocateInfo>(&import_info);
+    VkDeviceMemory imported_memory = VK_NULL_HANDLE;
+
+    // allocationSize != payload's allocationSize
+    {
+        alloc_info_with_import.allocationSize = payload_size * 2;
+        alloc_info_with_import.memoryTypeIndex = payload_memory_type;
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryAllocateInfo-allocationSize-01743");
+        vk::AllocateMemory(*m_device, &alloc_info_with_import, nullptr, &imported_memory);
+        m_errorMonitor->VerifyFound();
+    }
+    // memoryTypeIndex != payload's memoryTypeIndex
+    {
+        alloc_info_with_import.allocationSize = payload_size;
+        alloc_info_with_import.memoryTypeIndex = payload_memory_type + 1;
+        m_errorMonitor->SetDesiredFailureMsg(kErrorBit, "VUID-VkMemoryAllocateInfo-allocationSize-01743");
+        vk::AllocateMemory(*m_device, &alloc_info_with_import, nullptr, &imported_memory);
+        m_errorMonitor->VerifyFound();
+    }
+    // Importing memory object payloads from Windows handles does not transfer ownership of the handle to the driver.
+    // For NT handle types, the application must release handle ownership using the CloseHandle system call.
+    // That's in contrast with the POSIX file descriptor handles, where memory import operation transfers the ownership,
+    // so the application does not need to call 'close' system call.
+    ::CloseHandle(handle);
+}
+#endif
