@@ -12,7 +12,7 @@
  */
 
 #include "../framework/layer_validation_tests.h"
-#include "vk_extension_helper.h"
+#include "generated/vk_extension_helper.h"
 
 #include <algorithm>
 #include <array>
@@ -286,6 +286,7 @@ TEST_F(VkPositiveGraphicsLibraryLayerTest, ExeLibrary) {
     link_info.pLibraries = libraries;
 
     auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
+    exe_pipe_ci.layout = pre_raster_lib.gp_ci_.layout;
     vk_testing::Pipeline exe_pipe(*m_device, exe_pipe_ci);
     ASSERT_TRUE(exe_pipe.initialized());
 }
@@ -708,6 +709,7 @@ TEST_F(VkPositiveGraphicsLibraryLayerTest, DynamicPrimitiveTopolgy) {
     auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
     exe_pipe_ci.pInputAssemblyState = &ia_state;
     exe_pipe_ci.pDynamicState = &dynamic_create_info;
+    exe_pipe_ci.layout = layout;
     vk_testing::Pipeline exe_pipe(*m_device, exe_pipe_ci);
     ASSERT_TRUE(exe_pipe.initialized());
 
@@ -825,6 +827,121 @@ TEST_F(VkPositiveGraphicsLibraryLayerTest, LinkingInputAttachment) {
     link_info.pLibraries = libraries;
 
     auto exe_pipe_ci = LvlInitStruct<VkGraphicsPipelineCreateInfo>(&link_info);
+    exe_pipe_ci.layout = layout;
     vk_testing::Pipeline exe_pipe(*m_device, exe_pipe_ci);
     ASSERT_TRUE(exe_pipe.initialized());
-}
\ No newline at end of file
+}
+
+TEST_F(VkPositiveGraphicsLibraryLayerTest, TessellationWithoutPreRasterization) {
+    TEST_DESCRIPTION("have Tessellation stages with null pTessellationState but not Pre-Rasterization");
+
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    AddRequiredExtensions(VK_EXT_GRAPHICS_PIPELINE_LIBRARY_EXTENSION_NAME);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (DeviceValidationVersion() < VK_API_VERSION_1_2) {
+        GTEST_SKIP() << "At least Vulkan version 1.2 is required";
+    }
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
+
+    auto gpl_features = LvlInitStruct<VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT>();
+    GetPhysicalDeviceFeatures2(gpl_features);
+    if (!gpl_features.graphicsPipelineLibrary) {
+        GTEST_SKIP() << "VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT::graphicsPipelineLibrary not supported";
+    }
+
+    ASSERT_NO_FATAL_FAILURE(InitState(nullptr, &gpl_features));
+    if (!m_device->phy().features().tessellationShader) {
+        GTEST_SKIP() << "Device does not support tessellation shaders";
+    }
+
+    CreatePipelineHelper pipe(*this);
+    pipe.InitVertexInputLibInfo();
+    pipe.InitState();
+
+    VkPipelineShaderStageCreateInfo stages[2];
+
+    const auto tcs_spv = GLSLToSPV(VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT, bindStateTscShaderText);
+    auto tcs_ci = LvlInitStruct<VkShaderModuleCreateInfo>();
+    tcs_ci.codeSize = tcs_spv.size() * sizeof(decltype(tcs_spv)::value_type);
+    tcs_ci.pCode = tcs_spv.data();
+    stages[0] = LvlInitStruct<VkPipelineShaderStageCreateInfo>(&tcs_ci);
+    stages[0].stage = VK_SHADER_STAGE_TESSELLATION_CONTROL_BIT;
+    stages[0].module = VK_NULL_HANDLE;
+    stages[0].pName = "main";
+
+    const auto tes_spv = GLSLToSPV(VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT, bindStateTeshaderText);
+    auto tes_ci = LvlInitStruct<VkShaderModuleCreateInfo>();
+    tes_ci.codeSize = tes_spv.size() * sizeof(decltype(tes_spv)::value_type);
+    tes_ci.pCode = tes_spv.data();
+    stages[1] = LvlInitStruct<VkPipelineShaderStageCreateInfo>(&tes_ci);
+    stages[1].stage = VK_SHADER_STAGE_TESSELLATION_EVALUATION_BIT;
+    stages[1].module = VK_NULL_HANDLE;
+    stages[1].pName = "main";
+
+    pipe.gp_ci_.stageCount = 2;
+    pipe.gp_ci_.pStages = stages;
+    ASSERT_VK_SUCCESS(pipe.CreateGraphicsPipeline(true, false));
+}
+
+TEST_F(VkPositiveGraphicsLibraryLayerTest, FSIgnoredPointerGPLDynamicRendering) {
+    TEST_DESCRIPTION("Check ignored pointers with dynamics rendering and GPL");
+
+    AddRequiredExtensions(VK_EXT_GRAPHICS_PIPELINE_LIBRARY_EXTENSION_NAME);
+    AddRequiredExtensions(VK_KHR_DYNAMIC_RENDERING_EXTENSION_NAME);
+
+    SetTargetApiVersion(VK_API_VERSION_1_2);
+    ASSERT_NO_FATAL_FAILURE(InitFramework());
+    if (DeviceValidationVersion() < VK_API_VERSION_1_2) {
+        GTEST_SKIP() << "At least Vulkan version 1.2 is required";
+    }
+
+    if (!AreRequiredExtensionsEnabled()) {
+        GTEST_SKIP() << RequiredExtensionsNotSupported() << " not supported";
+    }
+
+    auto dynamic_rendering_features = LvlInitStruct<VkPhysicalDeviceDynamicRenderingFeaturesKHR>();
+    auto gpl_features = LvlInitStruct<VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT>(&dynamic_rendering_features);
+    GetPhysicalDeviceFeatures2(gpl_features);
+    if (!gpl_features.graphicsPipelineLibrary) {
+        GTEST_SKIP() << "VkPhysicalDeviceGraphicsPipelineLibraryFeaturesEXT::graphicsPipelineLibrary not supported";
+    }
+    if (!dynamic_rendering_features.dynamicRendering) {
+        GTEST_SKIP() << "Test requires (unsupported) dynamicRendering";
+    }
+
+    ASSERT_NO_FATAL_FAILURE(InitState(nullptr, &gpl_features));
+    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
+
+    CreatePipelineHelper fs_lib(*this);
+    {
+        VkStencilOpState stencil = {};
+        stencil.failOp = VK_STENCIL_OP_KEEP;
+        stencil.passOp = VK_STENCIL_OP_KEEP;
+        stencil.depthFailOp = VK_STENCIL_OP_KEEP;
+        stencil.compareOp = VK_COMPARE_OP_NEVER;
+
+        auto ds_ci = LvlInitStruct<VkPipelineDepthStencilStateCreateInfo>();
+        ds_ci.depthTestEnable = VK_FALSE;
+        ds_ci.depthWriteEnable = VK_TRUE;
+        ds_ci.depthCompareOp = VK_COMPARE_OP_NEVER;
+        ds_ci.depthBoundsTestEnable = VK_FALSE;
+        ds_ci.stencilTestEnable = VK_TRUE;
+        ds_ci.front = stencil;
+        ds_ci.back = stencil;
+
+        const auto fs_spv = GLSLToSPV(VK_SHADER_STAGE_FRAGMENT_BIT, bindStateFragShaderText);
+        vk_testing::GraphicsPipelineLibraryStage fs_stage(fs_spv, VK_SHADER_STAGE_FRAGMENT_BIT);
+
+        // Pass rendering info with null pointers that should be ignored
+        auto pipeline_rendering_info = LvlInitStruct<VkPipelineRenderingCreateInfo>();
+        pipeline_rendering_info.colorAttachmentCount = 2;  // <- bad data that should be ignored
+
+        fs_lib.InitFragmentLibInfo(1, &fs_stage.stage_ci, &pipeline_rendering_info);
+        fs_lib.InitState();
+        fs_lib.gp_ci_.renderPass = VK_NULL_HANDLE;
+        fs_lib.gp_ci_.pDepthStencilState = &ds_ci;
+        ASSERT_VK_SUCCESS(fs_lib.CreateGraphicsPipeline());
+    }
+}
